package ru.yandex.direct.useractionlog.model;

import java.util.Objects;

/**
 * Настройки для которых возможно автоматическое применение рекомендаций
 */
public class AutoChangeableSettings {

    private String type;

    private String item;

    private String subitem;

    private String recommendationOptionName;

    /**
     * Тип объекта (кампания, опция кампании и т.п.) для поиска в логах */
    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    public AutoChangeableSettings withType(String type) {
        setType(type);
        return this;
    }

    /**
     * Поле объекта (стратегия, настройки дневного бюджета и т.п.) для поиска в логах */
    public String getItem() {
        return item;
    }

    public void setItem(String item) {
        this.item = item;
    }

    public AutoChangeableSettings withItem(String item) {
        setItem(item);
        return this;
    }

    /**
     * Параметр внутри поля объекта (недельный бюджет внутри стратегии) */
    public String getSubitem() {
        return subitem;
    }

    public void setSubitem(String subitem) {
        this.subitem = subitem;
    }

    public AutoChangeableSettings withSubitem(String subitem) {
        setSubitem(subitem);
        return this;
    }

    /**
     * Название опции кампании включение которой позволяет автоматически изменять указанное поле/параметр */
    public String getRecommendationOptionName() {
        return recommendationOptionName;
    }

    public void setRecommendationOptionName(String recommendationOptionName) {
        this.recommendationOptionName = recommendationOptionName;
    }

    public AutoChangeableSettings withRecommendationOptionName(String recommendationOptionName) {
        setRecommendationOptionName(recommendationOptionName);
        return this;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder("AutoChangeableSettings{");
        sb.append("type=").append(type);
        sb.append(", item=").append(item);
        sb.append(", subitem=").append(subitem);
        sb.append(", recommendationOptionName=").append(recommendationOptionName);
        sb.append('}');
        return sb.toString();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        AutoChangeableSettings that = (AutoChangeableSettings) o;
        return Objects.equals(type, that.type)
                && Objects.equals(item, that.item)
                && Objects.equals(subitem, that.subitem)
                && Objects.equals(recommendationOptionName, that.recommendationOptionName);
    }

    @Override
    public int hashCode() {
        return Objects.hash(
                type,
                item,
                subitem,
                recommendationOptionName
        );
    }
}
