package ru.yandex.autotests.direct.api.campaigns.add.dynamictextcampaign.strategy.maximumconversionrate;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Collection;

import com.yandex.direct.api.v5.campaigns.DynamicTextCampaignSearchStrategyTypeEnum;
import org.junit.Before;
import org.junit.ClassRule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.api.campaigns.CampaignsFeatures;
import ru.yandex.autotests.directapi.apiclient.errors.Api5ErrorDetails;
import ru.yandex.autotests.directapi.model.api5.campaigns.AddRequestMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.CampaignAddItemMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.DynamicTextCampaignAddItemMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.DynamicTextCampaignNetworkStrategyAddMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.DynamicTextCampaignSearchStrategyAddMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.DynamicTextCampaignStrategyAddMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.PriorityGoalsArrayMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.PriorityGoalsItemMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.StrategyMaximumConversionRateAddMap;
import ru.yandex.autotests.directapi.model.api5.general.ExpectedResult;
import ru.yandex.autotests.directapi.model.api5.general.Notification;
import ru.yandex.autotests.directapi.model.campaigns.MetrikaGoals;
import ru.yandex.autotests.directapi.steps.ApiSteps;
import ru.yandex.direct.currency.Currency;
import ru.yandex.direct.currency.CurrencyCode;
import ru.yandex.direct.currency.Money;
import ru.yandex.qatools.allure.annotations.Description;
import ru.yandex.qatools.allure.annotations.Features;

import static ru.yandex.autotests.directapi.model.api5.campaigns.StrategyMaximumConversionRateAddMap.MEANINGFUL_GOAL_OPTIMIZATION_GOAL_ID;


@Aqua.Test
@Features(CampaignsFeatures.ADD)
@RunWith(Parameterized.class)
@Description("Негативные сценарии проверки параметра GoalId при создании кампании со стратегией " +
        "WbMaximumConversionRate")
public class StrategyMaximumConversionRateMeaningfulGoalIdNegativeTest {
    @ClassRule
    public static final ApiSteps api = new ApiSteps().asUnknownClient();
    private static final Currency currency = CurrencyCode.RUB.getCurrency();
    private static final Money weeklyLimit = Money.valueOf(
            currency.getMaxPrice().multiply(BigDecimal.valueOf(2)),
            currency.getCode());
    private static final Long TEST_MEANINGFUL_GOAL_VALUE = 13_000_000L;

    @Parameterized.Parameter(value = 0)
    public String description;
    @Parameterized.Parameter(value = 1)
    public PriorityGoalsArrayMap prioiityGoals;
    @Parameterized.Parameter(value = 2)
    public ExpectedResult expectedResult;
    private DynamicTextCampaignStrategyAddMap strategyMap;

    @Parameterized.Parameters(name = "test = {0}")
    public static Collection<?> priorityGoals() {
        Object[][] data = new Object[][]{
                {"КЦ не заданы", null,
                        ExpectedResult.errors(new Notification(4000,
                                Api5ErrorDetails.MAX_CONVERSION_RATE_MEANINGFUL_GOALS_IS_NOT_SET))},
                {"КЦ вовлеченные сессии", new PriorityGoalsArrayMap().withItems(
                        new PriorityGoalsItemMap()
                                .withEngagedSessionGoalId()
                                .withValue(TEST_MEANINGFUL_GOAL_VALUE)),
                        ExpectedResult.errors(new Notification(4000,
                                Api5ErrorDetails.MAX_CONVERSION_RATE_MEANINGFUL_GOALS_IS_NOT_SET))},
        };
        return Arrays.asList(data);
    }

    @Before
    public void clearUnits() {
        strategyMap = new DynamicTextCampaignStrategyAddMap()
                .withSearch(new DynamicTextCampaignSearchStrategyAddMap()
                        .withBiddingStrategyType(DynamicTextCampaignSearchStrategyTypeEnum.WB_MAXIMUM_CONVERSION_RATE)
                        .withWbMaximumConversionRate(new StrategyMaximumConversionRateAddMap()
                                .withWeeklySpendLimit(weeklyLimit.micros())
                                .withGoalId(MEANINGFUL_GOAL_OPTIMIZATION_GOAL_ID)))
                .withNetwork(new DynamicTextCampaignNetworkStrategyAddMap().defaultServingOff());
    }

    @Test
    public void goalIdNegativeTest() {
        api.userSteps.campaignSteps().shouldGetResultOnAdd(
                new AddRequestMap().withCampaigns(new CampaignAddItemMap()
                        .defaultCampaignAddItem()
                        .withDynamicTextCampaign(new DynamicTextCampaignAddItemMap()
                                .withCounterIds(MetrikaGoals.VALID_COUNTER_ID)
                                .withPriorityGoals(prioiityGoals)
                                .withBiddingStrategy(strategyMap))),
                expectedResult);
    }
}
