package ru.yandex.direct.core.copyentity.preprocessors;

import javax.annotation.ParametersAreNonnullByDefault;

import org.springframework.stereotype.Component;

import ru.yandex.direct.core.copyentity.CopyOperationContainer;
import ru.yandex.direct.core.entity.currency.service.CurrencyConverterFactory;
import ru.yandex.direct.core.entity.dynamictextadtarget.model.DynamicAdTarget;
import ru.yandex.direct.currency.Currencies;
import ru.yandex.direct.currency.Currency;
import ru.yandex.direct.currency.CurrencyCode;

@Component
@ParametersAreNonnullByDefault
public class DynamicAdTargetCopyPreprocessor implements CopyPreprocessorTypeSupport<DynamicAdTarget> {
    private final CurrencyConverterFactory currencyConverterFactory;

    public DynamicAdTargetCopyPreprocessor(CurrencyConverterFactory currencyConverterFactory) {
        this.currencyConverterFactory = currencyConverterFactory;
    }

    @Override
    public Class<DynamicAdTarget> getTypeClass() {
        return DynamicAdTarget.class;
    }

    @Override
    public void preprocess(DynamicAdTarget entity, CopyOperationContainer copyContainer) {
        entity.setDynamicConditionId(null);
        convertPrices(entity, copyContainer);
    }

    private void convertPrices(DynamicAdTarget entity, CopyOperationContainer copyContainer) {
        CurrencyCode currencyCodeFrom = copyContainer.getClientFrom().getWorkCurrency();
        CurrencyCode currencyCodeTo = copyContainer.getClientTo().getWorkCurrency();
        Currency currencyTo = Currencies.getCurrency(currencyCodeTo);

        if (currencyCodeFrom != currencyCodeTo) {
            // смена валюты возможна только при копировании кампании, то есть стратегия либо не меняется, либо
            // меняется на автобюджет, при которой задавать ставки по-умолчанию не нужно, поэтому просто
            // конвертируем те что есть.
            var converter = currencyConverterFactory.createConverter(currencyCodeFrom, currencyCodeTo);

            var minPrice = currencyTo.getMinPrice();
            var maxPrice = currencyTo.getMaxPrice();

            if (entity.getPrice() != null) {
                entity.setPrice(converter.convert(entity.getPrice(), minPrice, maxPrice));
            }

            if (entity.getPriceContext() != null) {
                entity.setPriceContext(converter.convert(entity.getPriceContext(), minPrice, maxPrice));
            }
        }
    }
}
