package ru.yandex.direct.core.entity.campaign.repository.type;

import java.util.Collection;

import javax.annotation.ParametersAreNonnullByDefault;

import org.jooq.Field;
import org.jooq.Record;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import ru.yandex.direct.core.entity.campaign.model.CampaignWithDisabledDomainsAndSsp;
import ru.yandex.direct.core.entity.campaign.repository.CampaignMappings;
import ru.yandex.direct.dbschema.ppc.tables.Campaigns;
import ru.yandex.direct.dbutil.wrapper.DslContextProvider;
import ru.yandex.direct.jooqmapper.JooqMapper;
import ru.yandex.direct.jooqmapper.JooqMapperBuilder;
import ru.yandex.direct.jooqmapperhelper.InsertHelperAggregator;
import ru.yandex.direct.jooqmapperhelper.UpdateHelperAggregator;
import ru.yandex.direct.model.AppliedChanges;

import static ru.yandex.direct.dbschema.ppc.Tables.CAMPAIGNS;
import static ru.yandex.direct.jooqmapper.ReaderWriterBuilders.convertibleProperty;
import static ru.yandex.direct.jooqmapper.ReaderWriterBuilders.property;

@Component
@ParametersAreNonnullByDefault
public class CampaignWithDisabledDomainsAndSspRepositoryTypeSupport extends AbstractCampaignRepositoryTypeSupport<CampaignWithDisabledDomainsAndSsp> {
    private static final JooqMapper<CampaignWithDisabledDomainsAndSsp> MAPPER =
            createCampaignWithContentLanguageMapper();

    @Autowired
    CampaignWithDisabledDomainsAndSspRepositoryTypeSupport(DslContextProvider dslContextProvider) {
        super(dslContextProvider);
    }

    private static JooqMapper<CampaignWithDisabledDomainsAndSsp> createCampaignWithContentLanguageMapper() {
        return JooqMapperBuilder.<CampaignWithDisabledDomainsAndSsp>builder()
                .map(property(CampaignWithDisabledDomainsAndSsp.ID, CAMPAIGNS.CID))
                .map(convertibleProperty(CampaignWithDisabledDomainsAndSsp.DISABLED_DOMAINS,
                        Campaigns.CAMPAIGNS.DONT_SHOW,
                        CampaignMappings::disabledDomainsFromDbToList, CampaignMappings::disabledDomainsToDb))
                .map(convertibleProperty(CampaignWithDisabledDomainsAndSsp.DISABLED_SSP,
                        Campaigns.CAMPAIGNS.DISABLED_SSP,
                        CampaignMappings::disabledSspFromJson, CampaignMappings::disabledSspToJson))
                .build();
    }

    @Override
    public void processUpdate(UpdateHelperAggregator updateHelperAggregator,
                              Collection<AppliedChanges<CampaignWithDisabledDomainsAndSsp>> appliedChanges) {
        updateHelperAggregator.getOrCreate(CAMPAIGNS.CID).processUpdateAll(MAPPER, appliedChanges);
    }

    @Override
    public void pushToInsert(InsertHelperAggregator insertHelperAggregator,
                             CampaignWithDisabledDomainsAndSsp campaign) {
        insertHelperAggregator.getOrCreate(CAMPAIGNS).add(MAPPER, campaign);
    }

    @Override
    public Collection<Field<?>> getFields() {
        return MAPPER.getFieldsToRead();
    }


    @Override
    public <M extends CampaignWithDisabledDomainsAndSsp> void fillFromRecord(M campaign, Record record) {
        MAPPER.fromDb(record, campaign);
    }

    @Override
    public Class<CampaignWithDisabledDomainsAndSsp> getTypeClass() {
        return CampaignWithDisabledDomainsAndSsp.class;
    }
}
