package ru.yandex.direct.grid.processing.service.campaign.loader;

import java.util.Map;
import java.util.concurrent.CompletableFuture;

import javax.annotation.ParametersAreNonnullByDefault;

import org.dataloader.MappedBatchLoaderWithContext;
import org.springframework.context.annotation.Scope;
import org.springframework.context.annotation.ScopedProxyMode;
import org.springframework.stereotype.Component;
import org.springframework.web.context.WebApplicationContext;

import ru.yandex.direct.core.entity.vcard.service.VcardService;
import ru.yandex.direct.dbutil.model.ClientId;
import ru.yandex.direct.grid.processing.context.container.GridGraphQLContext;
import ru.yandex.direct.grid.processing.model.client.GdClientInfo;
import ru.yandex.direct.grid.processing.service.dataloader.GridBatchingDataLoader;
import ru.yandex.direct.grid.processing.service.dataloader.GridContextProvider;

import static com.google.common.base.Preconditions.checkNotNull;

@Component
// DataLoader'ы хранят состояние, поэтому жить должны в рамках запроса
@Scope(value = WebApplicationContext.SCOPE_REQUEST, proxyMode = ScopedProxyMode.TARGET_CLASS)
@ParametersAreNonnullByDefault
public class CampaignsLastChangedVcardIdDataLoader extends GridBatchingDataLoader<Long, Long> {
    public CampaignsLastChangedVcardIdDataLoader(GridContextProvider gridContextProvider, VcardService vcardService) {
        this.dataLoader = mappedDataLoader(gridContextProvider, getBatchLoadFunction(vcardService));
    }

    private MappedBatchLoaderWithContext<Long, Long> getBatchLoadFunction(VcardService vcardService) {
        return (campaignsIds, environment) -> {
            GridGraphQLContext context = environment.getContext();
            GdClientInfo queriedClient = context.getQueriedClient();
            checkNotNull(queriedClient, "queriedClient should be set in gridContext");

            Map<Long, Long> lastVcardIdByCampaignId =
                    vcardService.getLastVcardIdByCampaignId(ClientId.fromLong(queriedClient.getId()), campaignsIds);

            return CompletableFuture.completedFuture(lastVcardIdByCampaignId);
        };
    }
}
