package ru.yandex.autotests.direct.api.ads.add.textad.videoextension;

import java.util.Arrays;
import java.util.Collection;

import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.api.ads.AdsFeatures;
import ru.yandex.autotests.direct.api.ads.AdsLogins;
import ru.yandex.autotests.direct.utils.model.ShardNumbers;
import ru.yandex.autotests.directapi.apiclient.config.Semaphore;
import ru.yandex.autotests.directapi.apiclient.errors.Api5ErrorDetails;
import ru.yandex.autotests.directapi.model.User;
import ru.yandex.autotests.directapi.model.api5.ads.AdAddItemMap;
import ru.yandex.autotests.directapi.model.api5.ads.AddRequestMap;
import ru.yandex.autotests.directapi.model.api5.ads.TextAdAddMap;
import ru.yandex.autotests.directapi.model.api5.general.ExpectedResult;
import ru.yandex.autotests.directapi.model.api5.general.Notification;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.qatools.allure.annotations.Description;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Issue;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

/**
 * https://st.yandex-team.ru/DIRECT-65827
 */
@Aqua.Test
@Features(AdsFeatures.ADD)
@Description("Проверка прав доступа агентства к CreativeId")
@Issue("https://st.yandex-team.ru/DIRECT-62100")
@RunWith(Parameterized.class)
public class AgencyCheckRightsToCreativeIdTest {

    private static String client = AdsLogins.CLIENT;
    private static String agency = AdsLogins.AGENCY;
    private static String agencyRep = AdsLogins.AGENCY_REP;
    private static String agencyRepCl = AdsLogins.AGENCY_REP_CL;
    private static String subclient = AdsLogins.SUBCLIENT;
    private static String subclientElse = AdsLogins.SUBCLIENT_ELSE;
    private static Long clientCreativeId;
    private static Long subclientCreativeId;
    private static Long subclientElseCreativeId;
    static Long adGroupId;

    @ClassRule
    public static ApiSteps api = new ApiSteps();

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    @Rule
    public Trashman trasher = new Trashman(api);

    @Parameterized.Parameter(0)
    public String loginForAuthorization;

    @Parameterized.Parameters(name = "loginForAuthorization = {0}")
    public static Collection testData() {
        Object[][] data = new Object[][]{
                {agency},
                {agencyRep},
                {agencyRepCl}
        };
        return Arrays.asList(data);
    }

    @BeforeClass
    public static void initTest() {
        api.userSteps.clientFakeSteps().fakeClearClientSpentUnits(client);
        api.userSteps.clientFakeSteps().fakeClearClientSpentUnits(agency);
        api.userSteps.clientFakeSteps().reshardUserWithLock(subclient, ShardNumbers.DEFAULT_SHARD);
        api.userSteps.clientFakeSteps().reshardUserWithLock(subclientElse, ShardNumbers.DEFAULT_SHARD);

        int shardId = api.userSteps.clientFakeSteps().getUserShard(client);
        clientCreativeId = api.userSteps.getDirectJooqDbSteps()
                .useShard(shardId).perfCreativesSteps().saveDefaultVideoCreative(
                        Long.parseLong(User.get(client).getClientID()), 1L);

        shardId = api.userSteps.clientFakeSteps().getUserShard(subclient);
        subclientCreativeId = api.userSteps.getDirectJooqDbSteps()
                .useShard(shardId).perfCreativesSteps().saveDefaultVideoCreative(
                        Long.parseLong(User.get(subclient).getClientID()), 1L);

        shardId = api.userSteps.clientFakeSteps().getUserShard(subclientElse);
        subclientElseCreativeId = api.userSteps.getDirectJooqDbSteps()
                .useShard(shardId).perfCreativesSteps().saveDefaultVideoCreative(
                        Long.parseLong(User.get(subclientElse).getClientID()), 1L);
        api.as(agency);
        Long subclientCampaignId = api.userSteps.campaignSteps().addDefaultTextCampaign(subclient);
        adGroupId = api.userSteps.adGroupsSteps().addDefaultGroup(subclientCampaignId, subclient);
    }

    @Test
    public void callAddWithSubclientCreativeId() {
        api.as(loginForAuthorization).userSteps.adsSteps().shouldGetResultOnAdd(
                new AddRequestMap().withAds(
                        new AdAddItemMap()
                                .withTextAd(new TextAdAddMap().defaultTextAd().withVideoExtension(subclientCreativeId))
                                .withAdGroupId(adGroupId)
                ),
                subclient,
                ExpectedResult.success()
        );
    }

    @Test
    public void callAddWithSubclientElseCreativeId() {
        api.as(loginForAuthorization).userSteps.adsSteps().shouldGetResultOnAdd(
                new AddRequestMap().withAds(
                        new AdAddItemMap()
                                .withTextAd(new TextAdAddMap().defaultTextAd().withVideoExtension(subclientElseCreativeId))
                                .withAdGroupId(adGroupId)
                ),
                subclient,
                ExpectedResult.errors(new Notification(8800, Api5ErrorDetails.VIDEO_ADDITION_NOT_FOUND))
        );
    }

    @Test
    public void callAddWithCreativeIdsOfBothSubclients() {
        api.as(loginForAuthorization).userSteps.adsSteps().shouldGetResultOnAdd(
                new AddRequestMap().withAds(
                        new AdAddItemMap()
                                .withTextAd(new TextAdAddMap().defaultTextAd().withVideoExtension(subclientCreativeId))
                                .withAdGroupId(adGroupId),
                        new AdAddItemMap()
                                .withTextAd(new TextAdAddMap().defaultTextAd().withVideoExtension(subclientElseCreativeId))
                                .withAdGroupId(adGroupId)
                ),
                subclient,
                ExpectedResult.success(),
                ExpectedResult.errors(new Notification(8800, Api5ErrorDetails.VIDEO_ADDITION_NOT_FOUND))
        );
    }

    @Test
    public void callAddWithClientCreativeId() {
        api.as(loginForAuthorization).userSteps.adsSteps().shouldGetResultOnAdd(
                new AddRequestMap().withAds(
                        new AdAddItemMap()
                                .withTextAd(new TextAdAddMap().defaultTextAd().withVideoExtension(clientCreativeId))
                                .withAdGroupId(adGroupId)
                ),
                subclient,
                ExpectedResult.errors(new Notification(8800, Api5ErrorDetails.VIDEO_ADDITION_NOT_FOUND))
        );
    }
}
