package ru.yandex.autotests.direct.api.ads.archive;

import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.api.ads.AdsFeatures;
import ru.yandex.autotests.direct.api.ads.AdsLogins;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.CampaignsStatusmoderate;
import ru.yandex.autotests.direct.utils.tags.TagDictionary;
import ru.yandex.autotests.directapi.apiclient.config.Semaphore;
import ru.yandex.autotests.directapi.model.User;
import ru.yandex.autotests.directapi.model.api5.ads.ArchiveRequestMap;
import ru.yandex.autotests.directapi.model.api5.general.ExpectedResult;
import ru.yandex.autotests.directapi.model.api5.general.IdsCriteriaMap;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.qatools.Tag;
import ru.yandex.qatools.allure.annotations.Description;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Issue;
import ru.yandex.qatools.allure.annotations.Step;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

/**
 * Created by semkagtn on 14.09.15.
 * https://st.yandex-team.ru/TESTIRT-6853
 */
@Aqua.Test
@Features(AdsFeatures.ARCHIVE)
@Description("Проверка архивации объявлений разных типов в одном запросе")
@Issue("https://st.yandex-team.ru/DIRECT-44779")
@Tag(TagDictionary.TRUNK)
public class ArchiveAdsOfDifferentTypesTest {

    private static final String LOGIN = AdsLogins.CLIENT41;

    @ClassRule
    public static ApiSteps api = new ApiSteps().as(LOGIN);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    @Rule
    public Trashman trasher = new Trashman(api);

    private static long textAdGroupId;
    private static long mobileAppAdGroupId;
    private static long dynamicTextAdGroupId;
    private static String adImageHash;


    @BeforeClass
    public static void createAdGroup() {
        api.userSteps.clientFakeSteps().fakeClearClientSpentUnits(LOGIN);

        Long textCampaignId = api.userSteps.campaignSteps().addDefaultTextCampaign();
        api.userSteps.getDirectJooqDbSteps().useShardForLogin(LOGIN).campaignsSteps()
                .setCampaignsStatusModerate(textCampaignId, CampaignsStatusmoderate.Yes);
        textAdGroupId = api.userSteps.adGroupsSteps().addDefaultGroup(textCampaignId);

        Long mobileCampaignId = api.userSteps.campaignSteps().addDefaultMobileAppCampaign();
        api.userSteps.getDirectJooqDbSteps().useShardForLogin(LOGIN).campaignsSteps()
                .setCampaignsStatusModerate(mobileCampaignId, CampaignsStatusmoderate.Yes);
        mobileAppAdGroupId = api.userSteps.adGroupsSteps().addDefaultMobileGroup(mobileCampaignId);

        Long dynamicTextCampaignId = api.userSteps.campaignSteps().addDefaultDynamicTextCampaign();
        api.userSteps.getDirectJooqDbSteps().useShardForLogin(LOGIN).campaignsSteps()
                .setCampaignsStatusModerate(dynamicTextCampaignId, CampaignsStatusmoderate.Yes);
        dynamicTextAdGroupId = api.userSteps.adGroupsSteps().addDefaultGroupDynamic(dynamicTextCampaignId);

        adImageHash = api.userSteps.imagesStepsV5().addImageAdAdImage(1);
    }

    private long textAdId;
    private long textImageAdId;
    private long mobileAppAdId;
    private long mobileAppImageAdId;
    private long mobileAppCpcVideoAdId;
    private long dynamicTextAdId;

    @Before
    @Step("Подготовка данных для теста")
    public void createAds() {
        api.userSteps.clientFakeSteps().fakeClearClientSpentUnits(LOGIN);

        Long creativeId = api.userSteps.getDirectJooqDbSteps()
                .useShardForLogin(LOGIN).perfCreativesSteps().saveDefaultCpcVideoCreative(
                        Long.parseLong(User.get(LOGIN).getClientID()));

        textAdId = api.userSteps.adsSteps().addDefaultTextAd(textAdGroupId);
        textImageAdId = api.userSteps.adsSteps().addDefaultTextImageAd(textAdGroupId, adImageHash);
        mobileAppAdId = api.userSteps.adsSteps().addDefaultMobileAppAd(mobileAppAdGroupId);
        mobileAppImageAdId = api.userSteps.adsSteps().addDefaultMobileAppImageAd(mobileAppAdGroupId, adImageHash);
        dynamicTextAdId = api.userSteps.adsSteps().addDefaultDynamicTextAd(dynamicTextAdGroupId);
        mobileAppCpcVideoAdId = api.userSteps.adsSteps()
                .addDefaultMobileAppCpcVideoAdBuilderAd(mobileAppAdGroupId, creativeId);

        api.userSteps.bannersFakeSteps().makeBannerFullyModerated(textAdId);
        api.userSteps.bannersFakeSteps().makeBannerFullyModerated(textImageAdId);
        api.userSteps.bannersFakeSteps().makeBannerFullyModerated(mobileAppAdId);
        api.userSteps.bannersFakeSteps().makeBannerFullyModerated(mobileAppImageAdId);
        api.userSteps.bannersFakeSteps().makeBannerFullyModerated(dynamicTextAdId);
        api.userSteps.bannersFakeSteps().makeBannerFullyModerated(mobileAppCpcVideoAdId);
    }

    @Test
    @Description("Архивация текстового объявления и объявления РМП за один запрос")
    public void archiveTextAdAndMobileAppAd() {
        api.userSteps.adsSteps().shouldGetResultOnArchive(new ArchiveRequestMap()
                        .withSelectionCriteria(new IdsCriteriaMap()
                                .withIds(textAdId, mobileAppAdId)),
                ExpectedResult.success(), ExpectedResult.success());
    }

    @Test
    @Description("Архивация текстового объявления и динамического текстового объявления за один запрос")
    public void archiveTextAdAndDynamicTextAd() {
        api.userSteps.adsSteps().shouldGetResultOnArchive(new ArchiveRequestMap()
                        .withSelectionCriteria(new IdsCriteriaMap()
                                .withIds(textAdId, dynamicTextAdId)),
                ExpectedResult.success(), ExpectedResult.success());
    }

    @Test
    @Description("Архивация текстового объявления, графического объявления, динамического текстового объявления, " +
            "объявления РМП и графического/видео объявлений РМП за один запрос")
    public void archiveTextAdAndTextImageAdAndDynamicTextAdAndMobileAppAdAndMobileAppImageAd() {
        api.userSteps.adsSteps().shouldGetResultOnArchive(new ArchiveRequestMap()
                        .withSelectionCriteria(new IdsCriteriaMap()
                                .withIds(textAdId, textImageAdId, dynamicTextAdId, mobileAppAdId, mobileAppImageAdId,
                                        mobileAppCpcVideoAdId)),
                ExpectedResult.success(), ExpectedResult.success(), ExpectedResult.success(), ExpectedResult.success(),
                ExpectedResult.success(), ExpectedResult.success());
    }
}
