package ru.yandex.autotests.direct.api.ads.archive;

import java.util.Arrays;
import java.util.Collection;

import org.junit.Before;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.api.ads.AccessRightsErrorHelper;
import ru.yandex.autotests.direct.api.ads.AdsFeatures;
import ru.yandex.autotests.direct.api.ads.AdsLogins;
import ru.yandex.autotests.direct.db.steps.DirectJooqDbSteps;
import ru.yandex.autotests.directapi.apiclient.config.Semaphore;
import ru.yandex.autotests.directapi.apiclient.errors.Api5Error;
import ru.yandex.autotests.directapi.apiclient.errors.Api5ErrorDetails;
import ru.yandex.autotests.directapi.model.Logins;
import ru.yandex.autotests.directapi.model.User;
import ru.yandex.autotests.directapi.model.api5.ads.AdAddItemMap;
import ru.yandex.autotests.directapi.model.api5.ads.ArchiveRequestMap;
import ru.yandex.autotests.directapi.model.api5.ads.CpcVideoAdBuilderAdAddMap;
import ru.yandex.autotests.directapi.model.api5.ads.TextAdAddMap;
import ru.yandex.autotests.directapi.model.api5.general.ExpectedResult;
import ru.yandex.autotests.directapi.model.api5.general.IdsCriteriaMap;
import ru.yandex.autotests.directapi.model.api5.general.Notification;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.qatools.allure.annotations.Description;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Issue;
import ru.yandex.qatools.allure.annotations.Step;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

@Aqua.Test
@Features(AdsFeatures.ARCHIVE)
@Description("Проверка архивации кликовых видеообъявлений от имени разных ролей")
@Issue("https://st.yandex-team.ru/DIRECT-82656")
@RunWith(Parameterized.class)
public class ArchiveCpcVideoAdRolesTest {

    @ClassRule
    public static ApiSteps api = new ApiSteps();

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    @Rule
    public Trashman trasher = new Trashman(api);

    @Parameterized.Parameter(0)
    public String description;

    @Parameterized.Parameter(1)
    public String apiUserLogin;

    @Parameterized.Parameter(2)
    public String login;

    @Parameterized.Parameter(3)
    public String loginToCreateAd;

    @Parameterized.Parameter(4)
    public ExpectedResult expectedResult;

    @Parameterized.Parameter(5)
    public Api5Error expectedJavaError;

    @Parameterized.Parameters(name = "{0}")
    public static Collection<Object[]> data() {
        return Arrays.asList(new Object[][]{
                {"Архивация объявления от имени менеджера", Logins.MANAGER_DEFAULT, AdsLogins.SERV_CLIENT,
                        AdsLogins.MANAGER_DEFAULT,
                        ExpectedResult.success(), null},

                {"Архивация объявления от имени вешальщика", Logins.PLACER, AdsLogins.CLIENT,
                        AdsLogins.CLIENT,
                        ExpectedResult.errors(new Notification(54, Api5ErrorDetails.NO_RIGHTS_TO_WRITE)),
                        new Api5Error(54)},

                {"Архивация объявления от имени саппорта", Logins.SUPPORT, AdsLogins.CLIENT,
                        AdsLogins.CLIENT,
                        ExpectedResult.success(), null},

                {"Архивация объявления от имени супера", Logins.SUPER_LOGIN, AdsLogins.CLIENT,
                        AdsLogins.CLIENT,
                        ExpectedResult.success(), null},

                {"Архивация объявления от имени медиапланера", Logins.MEDIA, AdsLogins.CLIENT,
                        AdsLogins.CLIENT,
                        ExpectedResult.errors(new Notification(54, Api5ErrorDetails.NO_RIGHTS_TO_WRITE)),
                        new Api5Error(54)},

                {"Архивация объявления от имени суперридера", Logins.SUPER_READER, AdsLogins.CLIENT,
                        AdsLogins.CLIENT,
                        ExpectedResult.errors(new Notification(54, Api5ErrorDetails.NO_RIGHTS_TO_WRITE)),
                        new Api5Error(54)},

                {"Архивация объявления от имени клиента для самого себя", AdsLogins.CLIENT, AdsLogins.CLIENT,
                        AdsLogins.CLIENT,
                        ExpectedResult.success(), null},

                {"Архивация объявления от имени представителя клиента для клиента", AdsLogins.CLIENT_REP, AdsLogins.CLIENT,
                        AdsLogins.CLIENT,
                        ExpectedResult.success(), null},

                {"Архивация объявления от имени агенства для субклиента", AdsLogins.AGENCY, AdsLogins.SUBCLIENT,
                        AdsLogins.AGENCY,
                        ExpectedResult.success(), null},

                {"Архивация объявления от имени представителя агенства для субклиента", AdsLogins.AGENCY_REP, AdsLogins.SUBCLIENT,
                        AdsLogins.AGENCY_REP,
                        ExpectedResult.success(), null},

                {"Архивация объявления от имени субклиента", AdsLogins.SUBCLIENT, AdsLogins.SUBCLIENT,
                        AdsLogins.AGENCY,
                        ExpectedResult.errors(new Notification(54, Api5ErrorDetails.NO_RIGHTS_TO_WRITE)),
                        new Api5Error(54)}
        });
    }

    private long adID;

    @Before
    @Step("Подготовка данных для теста")
    public void createAd() {
        api.userSteps.clientFakeSteps().fakeClearClientSpentUnits(apiUserLogin);

        api.as(loginToCreateAd);
        Long clientID = Long.parseLong(User.get(login).getClientID());
        Long campaignID = api.userSteps.campaignSteps().addDefaultTextCampaign(login);
        Long adGroupID = api.userSteps.adGroupsSteps().addDefaultGroup(campaignID, login);

        DirectJooqDbSteps jooqDbSteps =
                api.userSteps.getDarkSideSteps().getDirectJooqDbSteps().useShardForLogin(login);

        Long creativeId = jooqDbSteps.perfCreativesSteps().saveDefaultCpcVideoCreative(clientID);
        adID = api.userSteps.adsSteps().addDefaultCpcVideoAdBuilderAd(adGroupID, creativeId, login);
        api.userSteps.bannersFakeSteps().makeBannerFullyModerated(adID);
    }

    @Test
    public void requestUsingLogin() {
        api.as(apiUserLogin);
        AccessRightsErrorHelper.checkCatchingJavaError54(expectedJavaError, () -> api.userSteps.adsSteps()
                .shouldGetResultOnArchive(new ArchiveRequestMap().withSelectionCriteria(new IdsCriteriaMap()
                        .withIds(adID)), login, expectedResult));
    }
}
