package ru.yandex.autotests.direct.api.ads.update.dynamictextad;

import org.joda.time.DateTime;
import org.joda.time.LocalDate;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.api.ads.AdsFeatures;
import ru.yandex.autotests.direct.api.ads.AdsLogins;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.CampaignsStatusmoderate;
import ru.yandex.autotests.directapi.apiclient.config.Semaphore;
import ru.yandex.autotests.directapi.apiclient.errors.Api5ErrorDetails;
import ru.yandex.autotests.directapi.darkside.datacontainers.jsonrpc.fake.BannerFakeInfo;
import ru.yandex.autotests.directapi.darkside.model.Status;
import ru.yandex.autotests.directapi.model.api5.ads.AdAddItemMap;
import ru.yandex.autotests.directapi.model.api5.ads.AdUpdateItemMap;
import ru.yandex.autotests.directapi.model.api5.ads.DynamicTextAdAddMap;
import ru.yandex.autotests.directapi.model.api5.ads.DynamicTextAdUpdateMap;
import ru.yandex.autotests.directapi.model.api5.ads.UpdateRequestMap;
import ru.yandex.autotests.directapi.model.api5.general.ExpectedResult;
import ru.yandex.autotests.directapi.model.api5.general.Notification;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.qatools.allure.annotations.Description;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Issue;
import ru.yandex.qatools.allure.annotations.Step;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

import static org.apache.commons.lang3.StringUtils.capitalize;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.core.IsNot.not;
import static ru.yandex.autotests.irt.testutils.beandiffer.BeanDifferMatcher.beanEquivalent;

/**
 * Created by pavryabov on 01.12.15.
 * https://st.yandex-team.ru/TESTIRT-7862
 */
@Aqua.Test
@Features(AdsFeatures.UPDATE)
@Description("Проверка обновления статусов (StatusBsSynced и StatusModerate) при обновлении динамического текстового объявления")
@Issue("https://st.yandex-team.ru/DIRECT-43825")
public class UpdateDynamicTextAdStatusesTest {

    private static final String LOGIN = AdsLogins.CLIENT;

    @ClassRule
    public static ApiSteps api = new ApiSteps().as(LOGIN);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    @Rule
    public Trashman trasher = new Trashman(api);

    private static Long campaignId;
    private static Long adGroupId;
    private static long adId;
    private static DateTime lastChange = DateTime.now().minusDays(1);
    private static final DateTimeFormatter formatter = DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss");

    @BeforeClass
    public static void prepareData() {
        api.userSteps.clientFakeSteps().fakeClearClientSpentUnits(LOGIN);

        campaignId = api.userSteps.campaignSteps().addDefaultDynamicTextCampaign();
        api.userSteps.getDirectJooqDbSteps().useShardForLogin(LOGIN)
                .campaignsSteps().setCampaignsStatusModerate(campaignId, CampaignsStatusmoderate.Yes);
        adGroupId = api.userSteps.adGroupsSteps().addDefaultGroupDynamic(campaignId);
        api.userSteps.dynamicTextAdTargetsSteps().addDefaultWebpage(adGroupId);
        adId = api.userSteps.adsSteps().addAd(new AdAddItemMap()
                .withAdGroupId(adGroupId)
                .withDynamicTextAd(new DynamicTextAdAddMap()
                        .defaultDynamicTextAd()));
    }

    @Before
    @Step("Подготовка данных для теста")
    public void setAdStatuses() {
        api.userSteps.bannersFakeSteps().makeBannerFullyModerated(adId);
        api.userSteps.bannersFakeSteps().setStatusBsSynced(adId, Status.YES);
        api.userSteps.bannersFakeSteps().setLastChange(adId, lastChange.toString());

        api.userSteps.clientFakeSteps().fakeClearClientSpentUnits(LOGIN);
    }

    @Test
    @Description("Проверка обновления статусов при обновлении текста объявления")
    public void updateTextField() {
        api.userSteps.adsSteps().adsUpdate(new UpdateRequestMap()
                .withAds(new AdUpdateItemMap()
                        .withId(adId)
                        .withDynamicTextAd(new DynamicTextAdUpdateMap()
                                .withText("NEW TEXT"))));

        BannerFakeInfo actualBannerFakeInfo = api.userSteps.bannersFakeSteps().getBannerParams(adId);
        BannerFakeInfo expectedBannerFakeInfo = new BannerFakeInfo();
        expectedBannerFakeInfo.setStatusBsSynced(Status.NO);
        expectedBannerFakeInfo.setStatusModerate(Status.READY);

        assertThat("статусы объявления обновлись верно",
                actualBannerFakeInfo, beanEquivalent(expectedBannerFakeInfo));

        LocalDate actualDate = formatter.parseLocalDate(actualBannerFakeInfo.getLastChange());
        assertThat("LastChange объявления обновился", actualDate, equalTo(LocalDate.now()));
    }

    @Test
    @Description("Проверка необновления статусов при неудавшемся обновлении")
    public void updateWithError() {
        api.userSteps.adsSteps().shouldGetResultOnUpdate(new UpdateRequestMap()
                        .withAds(new AdUpdateItemMap()
                                .withId(adId)
                                .withDynamicTextAd(new DynamicTextAdUpdateMap()
                                        .withText(""))),
                ExpectedResult.errors(new Notification(5000,
                        Api5ErrorDetails.NO_VALUE_IN_FIELD, capitalize(DynamicTextAdAddMap.TEXT))));

        BannerFakeInfo actualBannerFakeInfo = api.userSteps.bannersFakeSteps().getBannerParams(adId);
        BannerFakeInfo expectedBannerFakeInfo = new BannerFakeInfo();
        expectedBannerFakeInfo.setStatusBsSynced(Status.YES);
        expectedBannerFakeInfo.setStatusModerate(Status.YES);

        assertThat("статусы объявления обновлись верно",
                actualBannerFakeInfo, beanEquivalent(expectedBannerFakeInfo));

        LocalDate actualDate = formatter.parseLocalDate(actualBannerFakeInfo.getLastChange());
        assertThat("LastChange объявления обновился", actualDate, not(equalTo(LocalDate.now())));
    }
}
