package ru.yandex.autotests.direct.api.audiencetargets.resume;

import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.api.audiencetargets.AudienceTargetsFeatures;
import ru.yandex.autotests.direct.api.audiencetargets.AudienceTargetsLogins;
import ru.yandex.autotests.directapi.apiclient.config.Semaphore;
import ru.yandex.autotests.directapi.apiclient.errors.Api5ErrorDetails;
import ru.yandex.autotests.directapi.model.api5.audiencetargets.ResumeRequestMap;
import ru.yandex.autotests.directapi.model.api5.general.ExpectedResult;
import ru.yandex.autotests.directapi.model.api5.general.IdsCriteriaMap;
import ru.yandex.autotests.directapi.model.api5.general.Notification;
import ru.yandex.autotests.directapi.model.retargeting.RetargetingGoalType;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.qatools.allure.annotations.Description;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Issue;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

/**
 * https://st.yandex-team.ru/TESTIRT-8519
 */
@Aqua.Test
@Features(AudienceTargetsFeatures.RESUME)
@Description("Проверка работы метода с одинаковыми Id в одном запросе")
@Issue("https://st.yandex-team.ru/DIRECT-48447")
public class ResumeAudienceTargetsSameIdsTest {
    private static final String LOGIN = AudienceTargetsLogins.LOGIN_TARGET39;

    @ClassRule
    public static ApiSteps api = new ApiSteps().as(LOGIN);

    @Rule
    public Trashman trashman = new Trashman(api);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    @BeforeClass
    public static void prepare() {
        api.userSteps.clientFakeSteps().fakeClearClientSpentUnits(LOGIN);
    }

    @Test
    public void testResumeWithTwoSameIds() {
        Long cid = api.userSteps.campaignSteps().addDefaultTextCampaign();
        Long pid = api.userSteps.adGroupsSteps().addDefaultGroup(cid);
        Long goalId = api.userSteps.retargetingSteps().getRetargetingGoalIDsByType(LOGIN, RetargetingGoalType.GOAL).get(0);
        Long retargetingListsId = api.userSteps.retargetingListsSteps().addDefaultRetargetingLists(goalId);
        Long targetId = api.userSteps.audienceTargetsSteps().addWithRetargetingList(pid, retargetingListsId);
        api.userSteps.audienceTargetsSteps().suspend(targetId);
        api.userSteps.audienceTargetsSteps().shouldGetResultOnResume(
                new ResumeRequestMap().withSelectionCriteria(new IdsCriteriaMap()
                        .withIds(targetId, targetId)),
                ExpectedResult.warnings(new Notification(10000, Api5ErrorDetails.RETARGETING_LISTS_ID_SHOULD_BE_ONCE_IN_REQUEST)),
                ExpectedResult.warnings(new Notification(10000, Api5ErrorDetails.RETARGETING_LISTS_ID_SHOULD_BE_ONCE_IN_REQUEST))
        );
    }
}
