package ru.yandex.autotests.direct.api.campaigns.add.notification;

import java.util.List;

import com.yandex.direct.api.v5.campaigns.CampaignGetItem;
import org.junit.Before;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.api.campaigns.CampaignsFeatures;
import ru.yandex.autotests.direct.api.campaigns.CampaignsLogins;
import ru.yandex.autotests.direct.utils.tags.TagDictionary;
import ru.yandex.autotests.directapi.apiclient.config.Semaphore;
import ru.yandex.autotests.directapi.model.api5.campaigns.CampaignAddItemMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.CampaignGetItemMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.EmailSettingsMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.NotificationMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.SmsSettingsMap;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.qatools.Tag;
import ru.yandex.qatools.allure.annotations.Description;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Issue;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

import static org.hamcrest.Matchers.hasSize;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assertThat;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assumeThat;
import static ru.yandex.autotests.irt.testutils.beandiffer.BeanDifferMatcher.beanEquivalent;

/**
 * Created by semkagtn on 03.09.15.
 * https://st.yandex-team.ru/TESTIRT-6546
 */
@Aqua.Test
@Features(CampaignsFeatures.ADD)
@Description("Проверка сохранения значений из структуры Notification в базе данных при создании: поля" +
        " WarningBalance, CheckPositionInterval, Email, TimeFrom, TimeTo")
@Issue("https://st.yandex-team.ru/DIRECT-34567")
@Tag(TagDictionary.TRUNK)
public class NotificationSaveToDatabaseSimpleFieldsTest {

    private static final String LOGIN = CampaignsLogins.SINGLE_CLIENT;

    @ClassRule
    public static ApiSteps api = new ApiSteps().as(LOGIN);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    @Rule
    public Trashman trasher = new Trashman(api);

    @Before
    public void addUnitsToClient() {
        api.userSteps.clientFakeSteps().fakeClearClientSpentUnits(LOGIN);
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("185")
    public void saveNotificationToDatabase() {
        NotificationMap notificationMap = new NotificationMap()
                .withEmailSettings(new EmailSettingsMap()
                        .withWarningBalance(EmailSettingsMap.MAX_WARNING_BALANCE / 5)
                        .withCheckPositionInterval(15)
                        .withEmail("some@email.ru"))
                .withSmsSettings(new SmsSettingsMap()
                        .withTimeFrom("11:00")
                        .withTimeTo("19:00"));

        long campaignId = api.userSteps.campaignSteps().addCampaign(new CampaignAddItemMap()
                .defaultCampaignAddItem()
                .withDefaultTextCampaign()
                .withNotification(notificationMap));

        List<CampaignGetItem> campaigns = api.userSteps.campaignSteps().getCampaigns(campaignId);
        assumeThat("была получена одна кампания", campaigns, hasSize(1));

        CampaignGetItem actualCampaign = campaigns.get(0);
        CampaignGetItem expectedCampaign = (CampaignGetItem) new CampaignGetItemMap()
                .withNotification(notificationMap).getBean();
        assertThat("значения сохранились верно", actualCampaign, beanEquivalent(expectedCampaign));
    }

    /**
     * Тест проверяет что пустая строка "" в качестве TimeFrom и TimeTo конвертируется в "00:00"
     * Эта фича не отражена в документации, но Директ Командер на неё расчитывает
     * Смотри так же https://st.yandex-team.ru/DIRECT-160850
     */
    @Test
    public void saveNotificationToDatabaseWithEmptyTimes() {
        NotificationMap notificationMap = new NotificationMap()
                .withEmailSettings(new EmailSettingsMap()
                        .withWarningBalance(EmailSettingsMap.MAX_WARNING_BALANCE / 5)
                        .withCheckPositionInterval(15)
                        .withEmail("some@email.ru"))
                .withSmsSettings(new SmsSettingsMap()
                        .withTimeFrom("")
                        .withTimeTo(""));

        long campaignId = api.userSteps.campaignSteps().addCampaign(new CampaignAddItemMap()
                .defaultCampaignAddItem()
                .withDefaultTextCampaign()
                .withNotification(notificationMap));

        List<CampaignGetItem> campaigns = api.userSteps.campaignSteps().getCampaigns(campaignId);
        assumeThat("была получена одна кампания", campaigns, hasSize(1));

        NotificationMap expectedNotificationMap = new NotificationMap()
                .withEmailSettings(new EmailSettingsMap()
                        .withWarningBalance(EmailSettingsMap.MAX_WARNING_BALANCE / 5)
                        .withCheckPositionInterval(15)
                        .withEmail("some@email.ru"))
                .withSmsSettings(new SmsSettingsMap()
                        .withTimeFrom("00:00")
                        .withTimeTo("00:00"));
        CampaignGetItem actualCampaign = campaigns.get(0);
        CampaignGetItem expectedCampaign = (CampaignGetItem) new CampaignGetItemMap()
                .withNotification(expectedNotificationMap).getBean();
        assertThat("значения сохранились верно", actualCampaign, beanEquivalent(expectedCampaign));
    }
}
