package ru.yandex.autotests.direct.api.campaigns.update;

import java.util.Arrays;

import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.api.campaigns.CampaignsFeatures;
import ru.yandex.autotests.direct.api.campaigns.CampaignsLogins;
import ru.yandex.autotests.direct.utils.tags.TagDictionary;
import ru.yandex.autotests.directapi.apiclient.config.Semaphore;
import ru.yandex.autotests.directapi.apiclient.errors.Api5Error;
import ru.yandex.autotests.directapi.apiclient.errors.Api5ErrorDetails;
import ru.yandex.autotests.directapi.model.api5.campaigns.CampaignUpdateItemMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.UpdateRequestMap;
import ru.yandex.autotests.directapi.model.api5.general.ExpectedResult;
import ru.yandex.autotests.directapi.model.api5.general.Notification;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.qatools.Tag;
import ru.yandex.qatools.allure.annotations.Description;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Issue;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

/**
 * Created by ginger on 23.08.15.
 * https://st.yandex-team.ru/TESTIRT-6809
 */
@Aqua.Test
@Issue("https://st.yandex-team.ru/DIRECT-34570")
@Features(CampaignsFeatures.UPDATE)
@Description("Проверка ограничения на кол-во одновременно изменяемых кампаний")
@Tag(TagDictionary.TRUNK)
public class UpdateCampaignsRestrictionTest {

    private static String client = CampaignsLogins.UPDATE_CLIENT;

    private static Long id;

    @ClassRule
    public static ApiSteps api = new ApiSteps().version(5).as(client);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    @Rule
    public Trashman trasher = new Trashman(api);

    @BeforeClass
    public static void createCampaign() {
        api.userSteps.clientFakeSteps().fakeClearClientSpentUnits(client);
        id = api.userSteps.campaignSteps().addDefaultTextCampaign();
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("896")
    public void updateMaxAmountOfCampaigns() {
        CampaignUpdateItemMap[] campaignUpdateMaps =
                new CampaignUpdateItemMap[UpdateRequestMap.MAX_AMOUNT_OF_CAMPAIGNS_IN_REQUEST];
        Arrays.fill(campaignUpdateMaps, new CampaignUpdateItemMap()
                .withId(id)
        );
        ExpectedResult[] expectedResults = new ExpectedResult[UpdateRequestMap.MAX_AMOUNT_OF_CAMPAIGNS_IN_REQUEST];
        Arrays.fill(expectedResults, ExpectedResult.errors(
                new Notification(9800, Api5ErrorDetails.CAMPAIGN_ID_IS_PRESENTED_MORE_THAN_ONCE_IN_REQUEST)));
        api.userSteps.campaignSteps().shouldGetResultOnUpdate(new UpdateRequestMap()
                        .withCampaigns(campaignUpdateMaps),
                expectedResults);
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("897")
    public void updateOverMaxAmountOfCampaigns() {
        CampaignUpdateItemMap[] campaignUpdateMaps =
                new CampaignUpdateItemMap[UpdateRequestMap.MAX_AMOUNT_OF_CAMPAIGNS_IN_REQUEST + 1];
        Arrays.fill(campaignUpdateMaps, new CampaignUpdateItemMap()
                .withId(id)
        );
        api.userSteps.campaignSteps().expectErrorOnCampaignsUpdate(
                new UpdateRequestMap()
                        .withCampaigns(campaignUpdateMaps),
                new Api5Error(9300,
                        Api5ErrorDetails.UPDATE_NO_MORE_THAN_CAMPAIGNS,
                        UpdateRequestMap.MAX_AMOUNT_OF_CAMPAIGNS_IN_REQUEST));
    }
}
