package ru.yandex.autotests.direct.api.campaigns.update.contentpromotion;

import com.yandex.direct.api.v5.campaignsext.ContentPromotionCampaignNetworkStrategyTypeEnum;
import com.yandex.direct.api.v5.campaignsext.ContentPromotionCampaignSearchStrategyTypeEnum;
import com.yandex.direct.api.v5.campaignsext.DailyBudgetModeEnum;
import com.yandex.direct.api.v5.general.YesNoEnum;
import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.api.campaigns.CampaignsFeatures;
import ru.yandex.autotests.direct.api.campaigns.CampaignsLogins;
import ru.yandex.autotests.direct.utils.money.Currency;
import ru.yandex.autotests.direct.utils.money.MoneyCurrency;
import ru.yandex.autotests.directapi.apiclient.config.Semaphore;
import ru.yandex.autotests.directapi.apiclient.errors.Api5ErrorDetails;
import ru.yandex.autotests.directapi.darkside.datacontainers.jsonrpc.fake.CampaignFakeInfo;
import ru.yandex.autotests.directapi.model.api5.campaignsext.CampaignAddItemMap;
import ru.yandex.autotests.directapi.model.api5.campaignsext.CampaignUpdateItemMap;
import ru.yandex.autotests.directapi.model.api5.campaignsext.ContentPromotionCampaignAddItemMap;
import ru.yandex.autotests.directapi.model.api5.campaignsext.ContentPromotionCampaignNetworkStrategyAddMap;
import ru.yandex.autotests.directapi.model.api5.campaignsext.ContentPromotionCampaignSearchStrategyAddMap;
import ru.yandex.autotests.directapi.model.api5.campaignsext.ContentPromotionCampaignSearchStrategyMap;
import ru.yandex.autotests.directapi.model.api5.campaignsext.ContentPromotionCampaignStrategyAddMap;
import ru.yandex.autotests.directapi.model.api5.campaignsext.ContentPromotionCampaignStrategyMap;
import ru.yandex.autotests.directapi.model.api5.campaignsext.ContentPromotionCampaignUpdateItemMap;
import ru.yandex.autotests.directapi.model.api5.campaignsext.DailyBudgetMap;
import ru.yandex.autotests.directapi.model.api5.campaignsext.EmailSettingsMap;
import ru.yandex.autotests.directapi.model.api5.campaignsext.NotificationMap;
import ru.yandex.autotests.directapi.model.api5.campaignsext.SmsSettingsMap;
import ru.yandex.autotests.directapi.model.api5.campaignsext.TimeTargetingMap;
import ru.yandex.autotests.directapi.model.api5.campaignsext.TimeTargetingOnPublicHolidaysMap;
import ru.yandex.autotests.directapi.model.api5.campaignsext.UpdateRequestMap;
import ru.yandex.autotests.directapi.model.api5.general.ExpectedResult;
import ru.yandex.autotests.directapi.model.api5.general.Notification;
import ru.yandex.autotests.directapi.model.campaigns.TimeTargetMap;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.autotests.irt.testutils.RandomUtils;
import ru.yandex.qatools.allure.annotations.Description;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Issue;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.not;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assumeThat;

/**
 * Created by pavryabov on 20.11.15.
 * https://st.yandex-team.ru/TESTIRT-7756
 */
@Aqua.Test
@Features(CampaignsFeatures.UPDATE)
@Description("Поверка LastChange после обновления динамической текстовой кампании")
@Issue("https://st.yandex-team.ru/DIRECT-44876")
public class CheckLastChangeAfterUpdateContentPromotionCampaignTest {

    private static final String CLIENT = CampaignsLogins.UPDATE_CLIENT;
    private static final String SSP_PLATFORM_DEFAULT = "Rubicon";
    private static final String DOMAIN = "qwerty.ru";

    @ClassRule
    public static ApiSteps api = new ApiSteps().version(5).as(CLIENT);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    @Rule
    public Trashman trasher = new Trashman(api);

    private static Long id;
    private String lastChange;

    @BeforeClass
    public static void initTest() {
        api.userSteps.getDirectJooqDbSteps().sspPlatformsSteps().saveSspPlatformsIfNotExist(SSP_PLATFORM_DEFAULT);
        api.userSteps.clientFakeSteps().fakeClearClientSpentUnits(CLIENT);
        id = api.userSteps.campExtSteps().addCampaign(new CampaignAddItemMap()
                .withName("name")
                .withStartDate(DateTime.now().toLocalDate().toString())
                .withContentPromotionCampaign(new ContentPromotionCampaignAddItemMap()
                        .withBiddingStrategy(new ContentPromotionCampaignStrategyAddMap()
                                .withSearch(new ContentPromotionCampaignSearchStrategyAddMap()
                                        .withBiddingStrategyType(
                                                ContentPromotionCampaignSearchStrategyTypeEnum.HIGHEST_POSITION))
                                .withNetwork(new ContentPromotionCampaignNetworkStrategyAddMap()
                                        .withBiddingStrategyType(
                                                ContentPromotionCampaignNetworkStrategyTypeEnum.SERVING_OFF)))));
    }

    @Before
    public void setLastChange() {
        lastChange = DateTime.now().minusMinutes(10).toString("yyyy-MM-dd hh:mm:ss");
        api.userSteps.campaignFakeSteps().setLastChange(id, lastChange);
        CampaignFakeInfo campaignFakeInfo = api.userSteps.campaignFakeSteps().fakeGetCampaignParams(id);
        assumeThat("LastChange у кампании сдвинут в прошлое", campaignFakeInfo.getLastChange(), equalTo(lastChange));
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("749")
    public void updateClientInfo() {
        api.userSteps.campExtSteps().shouldGetResultOnUpdate(new UpdateRequestMap()
                        .withCampaigns(new CampaignUpdateItemMap()
                                .withId(id)
                                .withClientInfo("someNewClientInfo")),
                ExpectedResult.success(id));
        CampaignFakeInfo campaignFakeInfo = api.userSteps.campaignFakeSteps().fakeGetCampaignParams(id);
        assumeThat("LastChange у кампании обновился",
                campaignFakeInfo.getLastChange(), not(equalTo(lastChange)));
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("750")
    public void updateNotification() {
        api.userSteps.campExtSteps().shouldGetResultOnUpdate(new UpdateRequestMap()
                        .withCampaigns(new CampaignUpdateItemMap()
                                .withId(id)
                                .withNotification(new NotificationMap()
                                        .withSmsSettings(new SmsSettingsMap()
                                                .withTimeFrom("20:00")
                                                .withTimeTo("20:15"))
                                        .withEmailSettings(new EmailSettingsMap().withEmail("somenewemail@mail.ru")))),
                ExpectedResult.success(id));
        CampaignFakeInfo campaignFakeInfo = api.userSteps.campaignFakeSteps().fakeGetCampaignParams(id);
        assumeThat("LastChange у кампании обновился",
                campaignFakeInfo.getLastChange(), not(equalTo(lastChange)));
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("751")
    public void updateEndDate() {
        api.userSteps.campExtSteps().shouldGetResultOnUpdate(new UpdateRequestMap()
                        .withCampaigns(new CampaignUpdateItemMap()
                                .withId(id)
                                .withEndDate(DateTime.now().plusDays(23).toLocalDate().toString())),
                ExpectedResult.success(id));
        CampaignFakeInfo campaignFakeInfo = api.userSteps.campaignFakeSteps().fakeGetCampaignParams(id);
        assumeThat("LastChange у кампании обновился",
                campaignFakeInfo.getLastChange(), not(equalTo(lastChange)));
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("752")
    public void updateStartDate() {
        api.userSteps.campExtSteps().shouldGetResultOnUpdate(new UpdateRequestMap()
                        .withCampaigns(new CampaignUpdateItemMap()
                                .withId(id)
                                .withStartDate(DateTime.now().plusDays(1).toLocalDate().toString())),
                ExpectedResult.success(id));
        CampaignFakeInfo campaignFakeInfo = api.userSteps.campaignFakeSteps().fakeGetCampaignParams(id);
        assumeThat("LastChange у кампании обновился",
                campaignFakeInfo.getLastChange(), not(equalTo(lastChange)));
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("753")
    public void updateTimeTargeting() {
        //DIRECT-45952
        api.userSteps.campExtSteps().shouldGetResultOnUpdate(new UpdateRequestMap()
                        .withCampaigns(new CampaignUpdateItemMap()
                                .withId(id)
                                .withTimeTargeting(new TimeTargetingMap()
                                        .withConsiderWorkingWeekends(YesNoEnum.YES)
                                        .withHolidaysSchedule(new TimeTargetingOnPublicHolidaysMap()
                                                .withSuspendOnHolidays(YesNoEnum.NO)
                                                .withStartHour(15)
                                                .withEndHour(16)))),
                ExpectedResult.success(id));
        CampaignFakeInfo campaignFakeInfo = api.userSteps.campaignFakeSteps().fakeGetCampaignParams(id);
        assumeThat("LastChange у кампании обновился",
                campaignFakeInfo.getLastChange(), not(equalTo(lastChange)));
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("754")
    public void updateTimeZone() {
        api.userSteps.campExtSteps().shouldGetResultOnUpdate(new UpdateRequestMap()
                        .withCampaigns(new CampaignUpdateItemMap()
                                .withId(id)
                                .withTimeZone(TimeTargetMap.TimeZone.ASIA_VIENTIANE)),
                ExpectedResult.success(id));
        CampaignFakeInfo campaignFakeInfo = api.userSteps.campaignFakeSteps().fakeGetCampaignParams(id);
        assumeThat("LastChange у кампании обновился",
                campaignFakeInfo.getLastChange(), not(equalTo(lastChange)));
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("755")
    public void updateNegativeKeywords() {
        api.userSteps.campExtSteps().shouldGetResultOnUpdate(new UpdateRequestMap()
                        .withCampaigns(new CampaignUpdateItemMap()
                                .withId(id)
                                .withNegativeKeywords("newNegativeKeyword")),
                ExpectedResult.success(id));
        CampaignFakeInfo campaignFakeInfo = api.userSteps.campaignFakeSteps().fakeGetCampaignParams(id);
        assumeThat("LastChange у кампании обновился",
                campaignFakeInfo.getLastChange(), not(equalTo(lastChange)));
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("756")
    public void updateBlockedIps() {
        api.userSteps.campExtSteps().shouldGetResultOnUpdate(new UpdateRequestMap()
                        .withCampaigns(new CampaignUpdateItemMap()
                                .withId(id)
                                .withBlockedIps("12.32.32.21")),
                ExpectedResult.success(id));
        CampaignFakeInfo campaignFakeInfo = api.userSteps.campaignFakeSteps().fakeGetCampaignParams(id);
        assumeThat("LastChange у кампании обновился",
                campaignFakeInfo.getLastChange(), not(equalTo(lastChange)));
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("757")
    public void updateExcludedSitesDomain() {
        api.userSteps.campExtSteps().shouldGetResultOnUpdate(new UpdateRequestMap()
                        .withCampaigns(new CampaignUpdateItemMap()
                                .withId(id)
                                .withExcludedSites(DOMAIN)),
                ExpectedResult.errors(new Notification(4000, Api5ErrorDetails.INVALID_FIELD_FOR_CAMP_TYPE,
                        "ExcludedSites")));
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("758")
    public void updateExcludedSitesSSPPlatform() {
        api.userSteps.campExtSteps().shouldGetResultOnUpdate(new UpdateRequestMap()
                        .withCampaigns(new CampaignUpdateItemMap()
                                .withId(id)
                                .withExcludedSites(SSP_PLATFORM_DEFAULT)),
                ExpectedResult.errors(new Notification(4000, Api5ErrorDetails.INVALID_FIELD_FOR_CAMP_TYPE,
                        "ExcludedSites")));
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("759")
    public void updateName() {

        api.userSteps.campExtSteps().shouldGetResultOnUpdate(new UpdateRequestMap()
                        .withCampaigns(new CampaignUpdateItemMap()
                                .withId(id)
                                .withName("someNewName")),
                ExpectedResult.success(id));
        CampaignFakeInfo campaignFakeInfo = api.userSteps.campaignFakeSteps().fakeGetCampaignParams(id);
        assumeThat("LastChange у кампании обновился",
                campaignFakeInfo.getLastChange(), not(equalTo(lastChange)));
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("760")
    public void updateDailyBudget() {
        api.userSteps.campExtSteps().shouldGetResultOnUpdate(new UpdateRequestMap()
                        .withCampaigns(new CampaignUpdateItemMap()
                                .withId(id)
                                .withDailyBudget(new DailyBudgetMap()
                                        .withAmount(
                                                MoneyCurrency.get(Currency.RUB).getMinDayBudgetAmount().bidLong().longValue())
                                        .withMode(DailyBudgetModeEnum.DISTRIBUTED))),
                ExpectedResult.success(id));
        CampaignFakeInfo campaignFakeInfo = api.userSteps.campaignFakeSteps().fakeGetCampaignParams(id);
        assumeThat("LastChange у кампании обновился",
                campaignFakeInfo.getLastChange(), not(equalTo(lastChange)));
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("761")
    public void updateCounterIds() {
        api.userSteps.campExtSteps().shouldGetResultOnUpdate(new UpdateRequestMap()
                        .withCampaigns(new CampaignUpdateItemMap()
                                .withId(id)
                                .withContentPromotionCampaign(new ContentPromotionCampaignUpdateItemMap()
                                        .withCounterIds(RandomUtils.getRandomInteger(0, 10000000)))),
                ExpectedResult.success(id));
        CampaignFakeInfo campaignFakeInfo = api.userSteps.campaignFakeSteps().fakeGetCampaignParams(id);
        assumeThat("LastChange у кампании обновился",
                campaignFakeInfo.getLastChange(), not(equalTo(lastChange)));
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("762")
    public void updateSearchStrategy() {
        api.userSteps.campExtSteps().shouldGetResultOnUpdate(new UpdateRequestMap()
                        .withCampaigns(new CampaignUpdateItemMap()
                                .withId(id)
                                .withContentPromotionCampaign(new ContentPromotionCampaignUpdateItemMap()
                                        .withBiddingStrategy(new ContentPromotionCampaignStrategyMap()
                                                .withSearch(new ContentPromotionCampaignSearchStrategyMap()
                                                        .withBiddingStrategyType(
                                                                ContentPromotionCampaignSearchStrategyTypeEnum.HIGHEST_POSITION))))),
                ExpectedResult.success(id));
        CampaignFakeInfo campaignFakeInfo = api.userSteps.campaignFakeSteps().fakeGetCampaignParams(id);
        assumeThat("LastChange у кампании обновился",
                campaignFakeInfo.getLastChange(), not(equalTo(lastChange)));
    }
}
