package ru.yandex.autotests.direct.api.campaigns.update.dynamictextcampaign.strategy.averagecpc;

import java.util.Arrays;
import java.util.Collection;

import com.yandex.direct.api.v5.campaigns.DynamicTextCampaignSearchStrategyTypeEnum;
import org.junit.Before;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.api.campaigns.CampaignsFeatures;
import ru.yandex.autotests.direct.api.campaigns.CampaignsLogins;
import ru.yandex.autotests.direct.utils.money.Currency;
import ru.yandex.autotests.direct.utils.money.Money;
import ru.yandex.autotests.direct.utils.money.MoneyCurrency;
import ru.yandex.autotests.direct.utils.money.MoneyFormat;
import ru.yandex.autotests.direct.utils.tags.TagDictionary;
import ru.yandex.autotests.directapi.apiclient.config.Semaphore;
import ru.yandex.autotests.directapi.apiclient.errors.Api5ErrorDetails;
import ru.yandex.autotests.directapi.apiclient.errors.Api5ErrorDetailsJava;
import ru.yandex.autotests.directapi.model.api5.campaigns.CampaignUpdateItemMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.DynamicTextCampaignNetworkStrategyMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.DynamicTextCampaignSearchStrategyMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.DynamicTextCampaignStrategyMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.DynamicTextCampaignUpdateItemMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.StrategyAverageCpcMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.UpdateRequestMap;
import ru.yandex.autotests.directapi.model.api5.general.ExpectedResult;
import ru.yandex.autotests.directapi.model.api5.general.JavaOrPerlExpectedResult;
import ru.yandex.autotests.directapi.model.api5.general.Notification;
import ru.yandex.autotests.directapi.model.campaigns.MetrikaGoals;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.qatools.Tag;
import ru.yandex.qatools.allure.annotations.Description;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Step;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

/**
 * Created by chicos on 07.09.2015.
 */
@Aqua.Test
@Features(CampaignsFeatures.UPDATE)
@RunWith(Parameterized.class)
@Description("Негативные сценарии проверки параметра AveragePrice при обновлении кампании со стратегией AverageCpc")
@Tag(TagDictionary.TRUNK)
public class StrategyAverageCpcAveragePriceNegativeTest {
    private static final String client = CampaignsLogins.CLIENT;
    private static final Currency currency = Currency.RUB;
    private static MoneyCurrency moneyCurrency = MoneyCurrency.get(currency);

    private static final Money minPrice = moneyCurrency.getMinAveragePrice();
    private static final Money maxPrice = moneyCurrency.getMaxPrice();

    private static final Integer counterId = MetrikaGoals.getCounterForLogin(client);
    private static final Long goalId = MetrikaGoals.getGoalForCounter(counterId);
    private Long campaignId;

    @ClassRule
    public static ApiSteps api = new ApiSteps().as(client);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    @Rule
    public Trashman trasher = new Trashman(api);

    @Parameterized.Parameter(value = 0)
    public String description;

    @Parameterized.Parameter(value = 1)
    public StrategyAverageCpcMap params;

    @Parameterized.Parameter(value = 2)
    public JavaOrPerlExpectedResult expectedResult;

    @Parameterized.Parameters(name = "test = {0}")
    public static Collection strategies() {
        JavaOrPerlExpectedResult expectedNotLess = JavaOrPerlExpectedResult
                .java(ExpectedResult.errors(
                        new Notification(4000,
                                Api5ErrorDetailsJava.VALUE_MUST_NOT_BE_LESS_WITHOUT_CURRENCY,
                                "AverageCpc",
                                minPrice.stringValue(MoneyFormat.WITHOUT_EXTRA_ZEROS))))
                .perl(ExpectedResult.errors(
                        new Notification(4000,
                                Api5ErrorDetails.AVERAGE_PRICE_SHOULD_BE_MORE_THAN,
                                minPrice.stringValue(MoneyFormat.TWO_DIGITS_POINT_SEPARATED),
                                moneyCurrency.getApiDescription())));
        Object[][] data = new Object[][]{
                //DIRECT-46025
                {"Нулевое значение", new StrategyAverageCpcMap().withAverageCpc(0L), expectedNotLess},
                {"Отрицательное значение", new StrategyAverageCpcMap().withAverageCpc(-1L), expectedNotLess},
                {"Значение меньше минимального",
                        new StrategyAverageCpcMap().withAverageCpc(minPrice.getPrevious().bidLong().longValue()),
                        expectedNotLess},
                {"Значение больше максимального",
                        new StrategyAverageCpcMap().withAverageCpc(maxPrice.getNext().bidLong().longValue()),
                        JavaOrPerlExpectedResult
                                .java(ExpectedResult.errors(new Notification(4000,
                                        Api5ErrorDetailsJava.VALUE_MUST_NOT_BE_GREATER_THAN,
                                        "AverageCpc",
                                        maxPrice.stringValue(MoneyFormat.WITHOUT_EXTRA_ZEROS))))
                                .perl(ExpectedResult.errors(new Notification(4000,
                                        Api5ErrorDetails.AVERAGE_PRICE_SHOULD_BE_LESS_THAN,
                                        maxPrice.stringValue(MoneyFormat.TWO_DIGITS_POINT_SEPARATED),
                                        moneyCurrency.getApiDescription())))},
                {"Среднее значение ставки больше недельного бюджета",
                        new StrategyAverageCpcMap()
                                .withAverageCpc(
                                        MoneyCurrency.get(currency).getMinWeeklyBudgetAmount().getNext().bidLong()
                                                .longValue())
                                .withWeeklySpendLimit(
                                MoneyCurrency.get(currency).getMinWeeklyBudgetAmount().bidLong().longValue()),
                        JavaOrPerlExpectedResult
                                .java(ExpectedResult.errors(new Notification(4000,
                                        Api5ErrorDetailsJava.MUST_BE_LESS_THAN_OR_EQUAL_TO_WEEKLY_BUDGET,
                                        "AverageCpc")))
                                .perl(ExpectedResult.errors(new Notification(4000,
                                        Api5ErrorDetails.WEEKLY_BUDGET_SHOULD_BE_GREATER_THAN_AVERAGE_PRICE)))},
        };
        return Arrays.asList(data);
    }

    @Before
    @Step("Подготовим данные для теста")
    public void clearUnits() {
        api.userSteps.clientFakeSteps().fakeClearClientSpentUnits(client);
        campaignId = api.userSteps.campaignSteps().addDefaultDynamicTextCampaign();
        int shard = api.userSteps.clientFakeSteps().getUserShard(client);
        api.userSteps.getDirectJooqDbSteps().useShard(shard).campaignsSteps()
                .addCampMetrikaGoals(campaignId, goalId, 100L, 100L);
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("972")
    public void averagePriceNegativeTest() {
        api.userSteps.campaignSteps().shouldGetResultOnUpdate(
                new UpdateRequestMap().withCampaigns(new CampaignUpdateItemMap()
                        .withId(campaignId)
                        .withDynamicTextCampaign(new DynamicTextCampaignUpdateItemMap()
                                .withBiddingStrategy(new DynamicTextCampaignStrategyMap()
                                        .withSearch(new DynamicTextCampaignSearchStrategyMap()
                                                .withBiddingStrategyType(
                                                        DynamicTextCampaignSearchStrategyTypeEnum.AVERAGE_CPC)
                                                .withAverageCpc(params))
                                        .withNetwork(
                                                new DynamicTextCampaignNetworkStrategyMap().defaultServingOff())))),
                expectedResult);
    }
}
