package ru.yandex.autotests.direct.api.campaigns.update.textcampaign.strategy.bids;

import java.math.BigInteger;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import com.yandex.direct.api.v5.keywords.KeywordGetItem;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.api.campaigns.CampaignsFeatures;
import ru.yandex.autotests.direct.api.campaigns.CampaignsLogins;
import ru.yandex.autotests.direct.db.models.jooq.ppc.tables.records.BidsManualPricesRecord;
import ru.yandex.autotests.direct.db.models.jooq.ppc.tables.records.BidsRecord;
import ru.yandex.autotests.direct.utils.money.Currency;
import ru.yandex.autotests.direct.utils.money.Money;
import ru.yandex.autotests.direct.utils.money.MoneyCurrency;
import ru.yandex.autotests.direct.utils.tags.TagDictionary;
import ru.yandex.autotests.directapi.apiclient.config.Semaphore;
import ru.yandex.autotests.directapi.model.api5.bids.BidSetItemMap;
import ru.yandex.autotests.directapi.model.api5.bids.SetRequestMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.CampaignAddItemMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.CampaignUpdateItemMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.TextCampaignAddItemMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.TextCampaignNetworkStrategyAddMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.TextCampaignNetworkStrategyMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.TextCampaignSearchStrategyAddMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.TextCampaignSearchStrategyMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.TextCampaignStrategyAddMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.TextCampaignStrategyMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.TextCampaignUpdateItemMap;
import ru.yandex.autotests.directapi.model.api5.keywords.KeywordGetItemMap;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.autotests.irt.testutils.allure.LogSteps;
import ru.yandex.qatools.Tag;
import ru.yandex.qatools.allure.annotations.Description;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Step;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

import static org.hamcrest.Matchers.hasSize;
import static org.hamcrest.Matchers.notNullValue;
import static ru.yandex.autotests.directapi.matchers.beans.version5.BeanDifferMatcherV5.beanEquivalentV5;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assertThat;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assumeThat;

/**
 * Created by chicos on 10.09.15.
 */
@Aqua.Test
@Features(CampaignsFeatures.UPDATE)
@RunWith(Parameterized.class)
@Description("При переключении стратегий кампании с ручной на автоматическую и обратно на ручную должны восстановиться первоначальные ставки ручной стратегии")
@Tag(TagDictionary.TRUNK)
public class StrategyRestoreManualBidsTest {
    private LogSteps log = LogSteps.getLogger(this.getClass());
    private static final String CLIENT = CampaignsLogins.CLIENT;
    private static final Currency currency = Currency.RUB;
    private static MoneyCurrency moneyCurrency = MoneyCurrency.get(currency);

    private static Integer userShard;
    private Long campaignId;
    private Long keywordId;
    private static final Money minPrice = moneyCurrency.getMinPrice();
    private static final Money contextBidToSet = minPrice.getNext().getNext();

    @ClassRule
    public static final ApiSteps api = new ApiSteps().as(CLIENT);

    @ClassRule
    public static final SemaphoreRule semaphore = Semaphore.getSemaphore();

    @Rule
    public Trashman trasher = new Trashman(api);

    @Parameterized.Parameter
    public String description;

    @Parameterized.Parameter(value = 1)
    public TextCampaignSearchStrategyAddMap searchStrategyPrevious;

    @Parameterized.Parameter(value = 2)
    public TextCampaignNetworkStrategyAddMap networkStrategyPrevious;

    @Parameterized.Parameter(value = 3)
    public TextCampaignSearchStrategyMap searchStrategy;

    @Parameterized.Parameter(value = 4)
    public TextCampaignNetworkStrategyMap networkStrategy;

    @Parameterized.Parameter(value = 5)
    public TextCampaignSearchStrategyMap searchStrategyFinal;

    @Parameterized.Parameter(value = 6)
    public TextCampaignNetworkStrategyMap networkStrategyFinal;

    @Parameterized.Parameter(value = 7)
    public Money bidToSet;

    @Parameterized.Parameter(value = 8)
    public Money expectedBid;

    @Parameterized.Parameter(value = 9)
    public Money expectedContextBid;

    @Parameterized.Parameters(name = "test = {0}")
    public static Collection strategies() {
        Object[][] data = new Object[][]{
                {"Ручная1 - Max -> Авто - Disabled -> Ручная1 - Max",
                        new TextCampaignSearchStrategyAddMap().defaultHighestPosition(),
                        new TextCampaignNetworkStrategyAddMap().defaultMaximumCoverage(),
                        new TextCampaignSearchStrategyMap().defaultWbMaximumClicks(currency),
                        new TextCampaignNetworkStrategyMap().defaultServingOff(),
                        new TextCampaignSearchStrategyMap().defaultHighestPosition(),
                        new TextCampaignNetworkStrategyMap().defaultMaximumCoverage(),
                        minPrice.getNext(),
                        minPrice.getNext(),
                        contextBidToSet},
                {"Ручная1 - Max -> Disabled - Авто -> Ручная1 - Max",
                        new TextCampaignSearchStrategyAddMap().defaultHighestPosition(),
                        new TextCampaignNetworkStrategyAddMap().defaultMaximumCoverage(),
                        new TextCampaignSearchStrategyMap().defaultServingOff(),
                        new TextCampaignNetworkStrategyMap().defaultWbMaximumClicks(currency),
                        new TextCampaignSearchStrategyMap().defaultHighestPosition(),
                        new TextCampaignNetworkStrategyMap().defaultMaximumCoverage(),
                        minPrice.getNext(),
                        minPrice.getNext(),
                        contextBidToSet},
        };
        return Arrays.asList(data);
    }

    @BeforeClass
    public static void getShard() {
        userShard = api.userSteps.clientFakeSteps().getUserShard(CLIENT);
    }

    @Before
    @Step("Подготовим данные для теста")
    public void clearUnits() {
        api.userSteps.clientFakeSteps().fakeClearClientSpentUnits(CLIENT);
        List<Long> ids = api.userSteps.campaignSteps().campaignsAddWithCheck(new CampaignAddItemMap()
                .defaultCampaignAddItem()
                .withTextCampaign(new TextCampaignAddItemMap()
                        .withBiddingStrategy(new TextCampaignStrategyAddMap()
                                .withSearch(searchStrategyPrevious)
                                .withNetwork(networkStrategyPrevious))));
        assumeThat("создана кампания", ids, hasSize(1));
        campaignId = ids.get(0);
        Long groupId = api.userSteps.adGroupsSteps().addDefaultGroup(campaignId);
        api.userSteps.adsSteps().addDefaultTextAd(groupId);
        keywordId = api.userSteps.keywordsSteps().addDefaultKeyword(groupId);

        //Установка ставок через сервис Bids
        api.userSteps.bidsSteps().bidsSet(new SetRequestMap().withBids(
                new BidSetItemMap()
                        .withKeywordId(keywordId)
                        .withBid(bidToSet.bidLong().longValue())
                        .withContextBid(contextBidToSet.bidLong().longValue())), null);
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("1162")
    public void bidChangeOnStrategyUpdateTest() {
        log.info("Обновим кампанию автостратегией");
        api.userSteps.campaignSteps().campaignsUpdateWithCheck(new CampaignUpdateItemMap()
                .withId(campaignId)
                .withTextCampaign(new TextCampaignUpdateItemMap()
                        .withBiddingStrategy(new TextCampaignStrategyMap()
                                .withSearch(searchStrategy)
                                .withNetwork(networkStrategy))));
        BidsManualPricesRecord priceReserve = api.userSteps.getDirectJooqDbSteps().useShard(userShard)
                .bidsManualPricesSteps().getBidsManualPrices(campaignId, BigInteger.valueOf(keywordId));
        assumeThat("ставки перенеслись в резерв таблицы bids_manual_prices", priceReserve, notNullValue());

        log.info("Выставим другие ставки для ключевого слова (иммитируем работу БК)");
        BidsRecord bid = api.userSteps.getDirectJooqDbSteps().useShard(userShard).bidsSteps()
                .getBidById(keywordId)
                .setPrice(expectedBid.getNext().bigDecimalValue())
                .setPriceContext(expectedContextBid.getNext().bigDecimalValue());
        api.userSteps.getDirectJooqDbSteps().bidsSteps().updateBids(bid);

        log.info("Вернем ручные стратегии");
        api.userSteps.campaignSteps().campaignsUpdateWithCheck(new CampaignUpdateItemMap()
                .withId(campaignId)
                .withTextCampaign(new TextCampaignUpdateItemMap()
                        .withBiddingStrategy(new TextCampaignStrategyMap()
                                .withSearch(searchStrategyFinal)
                                .withNetwork(networkStrategyFinal))));
        List<KeywordGetItem> keywords = api.userSteps.keywordsSteps().keywordsGetById(keywordId);
        assumeThat("получили ключевое слово", keywords, hasSize(1));

        KeywordGetItemMap expectedKeyword = new KeywordGetItemMap()
                .withBid(expectedBid.bidLong().longValue())
                .withContextBid(expectedContextBid.bidLong().longValue());

        assertThat("корректные ставки ключевого слова", keywords.get(0), beanEquivalentV5(expectedKeyword.getBean()));
    }
}
