package ru.yandex.autotests.direct.api.campaigns.update.timetargeting;

import com.yandex.direct.api.v5.campaigns.CampaignFieldEnum;
import com.yandex.direct.api.v5.campaigns.CampaignGetItem;
import com.yandex.direct.api.v5.campaigns.TimeTargetingOnPublicHolidays;
import com.yandex.direct.api.v5.general.YesNoEnum;
import org.junit.Before;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.api.campaigns.CampaignsFeatures;
import ru.yandex.autotests.direct.api.campaigns.CampaignsLogins;
import ru.yandex.autotests.directapi.apiclient.config.Semaphore;
import ru.yandex.autotests.directapi.model.api5.campaigns.CampaignAddItemMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.CampaignUpdateItemMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.TimeTargetingAddMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.TimeTargetingMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.TimeTargetingOnPublicHolidaysMap;
import ru.yandex.autotests.directapi.model.api5.campaigns.UpdateRequestMap;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.qatools.allure.annotations.Description;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Issue;
import ru.yandex.qatools.allure.annotations.Step;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

import static org.hamcrest.MatcherAssert.assertThat;
import static ru.yandex.autotests.directapi.matchers.beans.version5.BeanDifferMatcherV5.beanDifferV5;

/**
 * Created by lightelfik on 13.05.19.
 * https://st.yandex-team.ru/DIRECT-72103
 */
@Aqua.Test
@Features(CampaignsFeatures.UPDATE)
@Description("Проверка частичного обновления значений полей в структуре TimeTargeting")
@Issue("https://st.yandex-team.ru/DIRECT-34567")
public class TimeTargetingPartailUpdateTest {

    private static final String LOGIN = CampaignsLogins.SINGLE_CLIENT;

    TimeTargetingOnPublicHolidaysMap holidaysSchedule;

    @ClassRule
    public static ApiSteps api = new ApiSteps().as(LOGIN);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    @Rule
    public Trashman trasher = new Trashman(api);

    private long campaignId;

    @Before
    @Step("Подготовка данных для теста")
    public void addUnitsToClient() {
        api.userSteps.clientFakeSteps().fakeClearClientSpentUnits(LOGIN);

        holidaysSchedule = new TimeTargetingOnPublicHolidaysMap()
                .withSuspendOnHolidays(YesNoEnum.NO)
                .withStartHour(12)
                .withEndHour(22)
                .withBidPercent(90);

        campaignId = api.userSteps.campaignSteps().addCampaign(new CampaignAddItemMap()
                .defaultCampaignAddItem()
                .withDefaultTextCampaign()
                .withTimeTargeting(new TimeTargetingAddMap()
                        .withConsiderWorkingWeekends(YesNoEnum.NO)
                        .withHolidaysSchedule(holidaysSchedule)));
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("1204")
    public void partailUpdateHolidayScheduleTest() {

        api.userSteps.campaignSteps().campaignsUpdate(new UpdateRequestMap().withCampaigns(new CampaignUpdateItemMap()
                .withId(campaignId)
                .defaultCampaignUpdateItem()
                .withTimeTargeting(new TimeTargetingMap()
                        .withConsiderWorkingWeekends(YesNoEnum.YES))));

        CampaignGetItem campaign =
                api.userSteps.campaignSteps().campaignsGet(campaignId, CampaignFieldEnum.TIME_TARGETING);
        TimeTargetingOnPublicHolidays actualHolidaysSchedule =
                campaign.getTimeTargeting().getHolidaysSchedule().getValue();

        assertThat("значения структуры HolidaysSchedule были обновлены верно",
                actualHolidaysSchedule,
                beanDifferV5((TimeTargetingOnPublicHolidays) holidaysSchedule.getBean()));
    }
}
