package ru.yandex.autotests.directapi.dynamictextadtargets.resume;

import com.yandex.direct.api.v5.dynamictextadtargets.AddResponse;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.directapi.apiclient.config.Semaphore;
import ru.yandex.autotests.directapi.apiclient.errors.Api5ErrorDetails;
import ru.yandex.autotests.directapi.dynamictextadtargets.DynamicTextAdTargetsFeatures;
import ru.yandex.autotests.directapi.dynamictextadtargets.DynamicTextAdTargetsLogins;
import ru.yandex.autotests.directapi.model.api5.dynamictextadtargets.AddRequestMap;
import ru.yandex.autotests.directapi.model.api5.dynamictextadtargets.ResumeRequestMap;
import ru.yandex.autotests.directapi.model.api5.dynamictextadtargets.SuspendRequestMap;
import ru.yandex.autotests.directapi.model.api5.dynamictextadtargets.WebpageAddItemMap;
import ru.yandex.autotests.directapi.model.api5.general.ExpectedResult;
import ru.yandex.autotests.directapi.model.api5.general.IdsCriteriaMap;
import ru.yandex.autotests.directapi.model.api5.general.JavaOrPerlExpectedResult;
import ru.yandex.autotests.directapi.model.api5.general.Notification;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.qatools.allure.annotations.Description;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Issue;
import ru.yandex.qatools.allure.annotations.Step;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

import static org.hamcrest.Matchers.hasSize;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assumeThat;

/**
 * Created by ginger on 10.11.15.
 * https://st.yandex-team.ru/TESTIRT-6666
 */
@Aqua.Test
@Issue("https://st.yandex-team.ru/DIRECT-43826")
@Features(DynamicTextAdTargetsFeatures.RESUME)
@Description("Одинаковые идетификаторы в запросе на включение условия нацеливания")
public class ResumeSameIdsTest {
    private static final String LOGIN = DynamicTextAdTargetsLogins.SINGLE_CLIENT_RESUME;

    @ClassRule
    public static ApiSteps api = new ApiSteps().as(LOGIN);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();
    private static Long pid;
    private static AddResponse response;

    @Rule
    public Trashman trasher = new Trashman(api);

    @BeforeClass
    public static void initTest() {
        api.userSteps.clientFakeSteps().fakeClearClientSpentUnits(LOGIN);
    }

    @Before
    @Step("Подготовка данных для теста")
    public void addWebpage() {
        Long cid = api.as(LOGIN).userSteps.campaignSteps().addDefaultDynamicTextCampaign(LOGIN);
        pid = api.userSteps.adGroupsSteps().addDefaultGroupDynamic(cid);
        response =  api.userSteps.dynamicTextAdTargetsSteps().shouldGetResultOnAdd(
                new AddRequestMap()
                        .withWebpages(new WebpageAddItemMap()
                                .withAdGroupId(pid)
                                .withName("Name")
                                .withBid(30000000l)
                        ),
                ExpectedResult.success()
        );
        assumeThat("создалась динамическая группа с 1 Webpages", response.getAddResults(), hasSize(1));
        api.userSteps.dynamicTextAdTargetsSteps().dynamicTextAdTargetsSuspend(
                new SuspendRequestMap().withSelectionCriteria(new IdsCriteriaMap().withIds(response.getAddResults().get(0).getId()))
        ) ;

    }

    @Test
    @Description("Вызвать метод включения условий нацеливания для одинаковых ID")
    public void sameIdsInRequest() {
        JavaOrPerlExpectedResult bothWarning1 = JavaOrPerlExpectedResult.java(ExpectedResult.warnings(
                new Notification(10000, Api5ErrorDetails.WEBPAGE_ID_IN_REQUEST_MORE_THAN_ONCE)
                        .withDetails("Id условия присутствует в запросе более одного раза"))).perl(ExpectedResult
                .warnings(new Notification(10000, Api5ErrorDetails.WEBPAGE_ID_IN_REQUEST_MORE_THAN_ONCE)));
        JavaOrPerlExpectedResult bothWarning2 = JavaOrPerlExpectedResult.java(ExpectedResult.warnings(
                new Notification(10000, Api5ErrorDetails.WEBPAGE_ID_IN_REQUEST_MORE_THAN_ONCE)
                        .withDetails("Id условия присутствует в запросе более одного раза"))).perl(ExpectedResult
                .warnings(new Notification(10000, Api5ErrorDetails.WEBPAGE_ID_IN_REQUEST_MORE_THAN_ONCE),
                        new Notification(10243, Api5ErrorDetails.WEBPAGE_NOT_SUSPENDED)));
        // Разделение на две ошибки нужно убрать, когда приедут переводы в DIRECT-93168

        api.as(LOGIN).userSteps.dynamicTextAdTargetsSteps().shouldGetResultOnResume(
                new ResumeRequestMap()
                        .withSelectionCriteria(
                                new IdsCriteriaMap()
                                        .withIds(response.getAddResults().get(0).getId(), response.getAddResults().get(0).getId())),
                bothWarning1,
                bothWarning2);
    }
}
