package ru.yandex.autotests.directapi.reports.adreport;

import java.util.stream.Collectors;

import com.yandex.direct.api.v5.reports.FieldEnum;
import com.yandex.direct.api.v5.reports.ReportTypeEnum;
import org.junit.ClassRule;
import org.junit.Test;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.directapi.darkside.Logins;
import ru.yandex.autotests.directapi.model.api5.reports.ReportDefinitionMap;
import ru.yandex.autotests.directapi.model.api5.reports.ReportsData;
import ru.yandex.autotests.directapi.reports.ReportsFeatures;
import ru.yandex.autotests.directapi.reports.ReportsLogins;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.steps.statistics.ReportsSteps;
import ru.yandex.autotests.irt.testutils.beandiffer2.BeanDifferMatcher;
import ru.yandex.qatools.allure.annotations.Description;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Issue;

import static org.hamcrest.Matchers.containsInAnyOrder;
import static org.hamcrest.Matchers.empty;
import static org.hamcrest.Matchers.not;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assertThat;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assumeThat;

/**
 * Created by andy-ilyin on 13.01.17.
 */
@Aqua.Test
@Description("AD_PERFORMANCE_REPORT возвращает те же данные, что и CUSTOM_REPORT")
@Issue("https://st.yandex-team.ru/TESTIRT-10893")
@Features(ReportsFeatures.AD_PERFORMANCE_REPORT)
public class AdPerformanceReportCompareWithCustomReportTest {
    private static final long CAMPAIGN_ID = 20187436L;
    private static final String DATE_FROM = "2019-12-02";
    private static final String DATE_TO = "2019-12-08";

    private static final FieldEnum[] FIELD_NAMES = new FieldEnum[] {
            // по этому полю в этом типе отчёта всегда неявная группировка
            FieldEnum.AD_ID,

            FieldEnum.DATE,
            FieldEnum.CAMPAIGN_ID,
            FieldEnum.AD_GROUP_ID,
            FieldEnum.IMPRESSIONS,
            FieldEnum.CLICKS,
            FieldEnum.COST,
            FieldEnum.CTR,
            FieldEnum.TARGETING_CATEGORY,
            FieldEnum.INCOME_GRADE
    };

    @ClassRule
    public static final ApiSteps api = new ApiSteps().as(Logins.LOGIN_SUPER).clientLogin(ReportsLogins.CLIENT_FOR_AD_PERFORMANCE_REPORT1);

    private static final ReportsSteps REPORTS_STEPS = api.userSteps.reportsSteps();

    @Test
    public void test() {
        ReportsData adPerformanceReport = REPORTS_STEPS.callReportsXml(new ReportDefinitionMap()
                .withDefaultReportWithTypeAndUniqueReportName(ReportTypeEnum.AD_PERFORMANCE_REPORT)
                .withCustomDatesAndCampaignIds(DATE_FROM, DATE_TO, CAMPAIGN_ID)
                .withFieldNames(FIELD_NAMES)
                .withPage(null));

        assumeThat("отчёт AD_PERFORMANCE_REPORT не пустой",
                adPerformanceReport.getReportsLines(), not(empty()));

        ReportsData customReport = REPORTS_STEPS.callReportsXml(new ReportDefinitionMap()
                .withDefaultCustomReportWithUniqueReportName()
                .withCustomDatesAndCampaignIds(DATE_FROM, DATE_TO, CAMPAIGN_ID)
                .withFieldNames(FIELD_NAMES)
                .withPage(null));

        assertThat("в отчётах одни и те же данные",
                adPerformanceReport.getReportsLines(),
                containsInAnyOrder(customReport.getReportsLines().stream()
                        .map(BeanDifferMatcher::beanDiffer)
                        .collect(Collectors.toList())));
    }
}
