package ru.yandex.autotests.directintapi.bstransport.main.cpmbanner.transmit;

import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.cmd.rules.CreativeBannerRule;
import ru.yandex.autotests.direct.cmd.rules.DirectCmdRule;
import ru.yandex.autotests.direct.db.beans.adgroups.AdGroupPageBlock;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.BannersBannerType;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.BannersPerformanceStatusmoderate;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.ModerateBannerPagesStatusmoderate;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.ModerateBannerPagesStatusmoderateoperator;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.PerfCreativesCreativeType;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.PerfCreativesStatusmoderate;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.PhrasesAdgroupType;
import ru.yandex.autotests.direct.db.models.jooq.ppc.tables.records.BannersRecord;
import ru.yandex.autotests.direct.db.models.jooq.ppc.tables.records.PerfCreativesRecord;
import ru.yandex.autotests.direct.db.steps.DirectJooqDbSteps;
import ru.yandex.autotests.direct.utils.campaigns.CampaignTypeEnum;
import ru.yandex.autotests.direct.utils.tags.TagDictionary;
import ru.yandex.autotests.directapi.darkside.Logins;
import ru.yandex.autotests.directapi.darkside.connection.Semaphore;
import ru.yandex.autotests.directapi.darkside.datacontainers.jsonrpc.displaycanvas.ModerationInfo;
import ru.yandex.autotests.directapi.darkside.model.CampaignsType;
import ru.yandex.autotests.directapi.darkside.model.RunBsTransportScriptResponse;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Banner;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Campaign;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Context;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.PageModeration;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.autotests.directintapi.bstransport.FeatureNames;
import ru.yandex.autotests.directintapi.bstransport.TransportHelpSteps;
import ru.yandex.qatools.Tag;
import ru.yandex.qatools.allure.annotations.Description;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Issue;
import ru.yandex.qatools.allure.annotations.Step;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

import static java.util.Arrays.asList;
import static org.hamcrest.Matchers.contains;
import static org.hamcrest.core.IsNull.notNullValue;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assertThat;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assumeThat;
import static ru.yandex.autotests.irt.testutils.beandiffer2.BeanDifferMatcher.beanDiffer;

@Aqua.Test
@Tag(TagDictionary.RELEASE)
@Tag(TagDictionary.TRUNK)
@Description("Проверяем отправку полей модерации пейджей в cpm_outdoor баннере")
@Features(FeatureNames.CPM_OUTDOOR)
@Issue("https://st.yandex-team.ru/DIRECT-91438")
public class SendingOfCpmOutdoorBannerPageModerationTest {

    private static final int PAGE_ID_1 = 1;
    private static final int PAGE_ID_2 = 2;
    private static final int PAGE_ID_3 = 3;
    private static final int PAGE_ID_4 = 4;
    private static final int PAGE_ID_5 = 5;
    private static final int PAGE_ID_6 = 6;
    private static final int PAGE_ID_7 = 7;

    private static final String TASK_URL = "https://sanbox.yandex.ru/12345";

    private static final String LOGIN = Logins.LOGIN_TRANSPORT;

    @ClassRule
    public static ApiSteps api = new ApiSteps().as(LOGIN);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    @Rule
    public Trashman trashman = new Trashman(api);

    @ClassRule
    public static DirectCmdRule defaultClassRule = DirectCmdRule.defaultClassRule();

    public CreativeBannerRule bannersRule = new CreativeBannerRule(CampaignTypeEnum.TEXT).withUlogin(LOGIN);

    @Rule
    public DirectCmdRule cmdRule = DirectCmdRule.defaultRule().withRules(bannersRule);

    private static int shard;
    private Long cid;
    private Long pid;
    private Long bid;
    public TransportHelpSteps helpSteps;

    @BeforeClass
    public static void beforeClass() {
        shard = api.userSteps.clientFakeSteps().getUserShard(LOGIN);
    }

    @Before
    @Step("Подготовка тестовых данных")
    public void init() {
        DirectJooqDbSteps jooqDbSteps = api.userSteps.getDirectJooqDbSteps().useShard(shard);
        helpSteps = new TransportHelpSteps(api).useDbJooqSteps(jooqDbSteps);

        cid = bannersRule.getCampaignId();
        Long creativeId = bannersRule.getCreativeId();
        bid = bannersRule.getBannerId();
        pid = bannersRule.getGroupId();
        Long retargetingConditionId = helpSteps.addRetargetingListsForUserIfNotExists(LOGIN).get(0);
        api.userSteps.audienceTargetsSteps().addWithRetargetingList(pid, retargetingConditionId);

        api.userSteps.campaignFakeSteps().makeNewCampaignReadyForSendingToBS(cid);
        api.userSteps.groupFakeSteps().makeGroupFullyModerated(pid);
        api.userSteps.bannersFakeSteps().makeBannerFullyModerated(bid);

        jooqDbSteps.bannersPerformanceSteps()
                .setCreativeStatusModerate(cid, pid, bid, BannersPerformanceStatusmoderate.Yes);
        PerfCreativesRecord perfCreativesRecord = jooqDbSteps.perfCreativesSteps().getPerfCreatives(creativeId);
        perfCreativesRecord.setModerateInfo(new ModerationInfo().generateTestData().toString());
        perfCreativesRecord.setStatusmoderate(PerfCreativesStatusmoderate.Yes);
        perfCreativesRecord.setCreativeType(PerfCreativesCreativeType.video_addition);
        jooqDbSteps.perfCreativesSteps().updatePerfCreatives(perfCreativesRecord);

        api.userSteps.campaignFakeSteps().setType(cid, CampaignsType.CPM_BANNER);
        jooqDbSteps.adGroupsSteps().setType(pid, PhrasesAdgroupType.cpm_outdoor);
        BannersRecord bannersRecord = jooqDbSteps.bannersSteps().getBanner(bid);
        bannersRecord.setBannerType(BannersBannerType.cpm_outdoor);
        bannersRecord.setTitle("");
        bannersRecord.setBody("");
        jooqDbSteps.bannersSteps().updateBanners(bannersRecord);

        jooqDbSteps.adGroupsSteps()
                .addAdGroupPageTargets(pid, asList(new AdGroupPageBlock().withPageId(PAGE_ID_1).withImpId(10),
                        new AdGroupPageBlock().withPageId(PAGE_ID_2).withImpId(11),
                        new AdGroupPageBlock().withPageId(PAGE_ID_3).withImpId(12)));
        //добавляем вердикты
        //отправляем только statusModerate = Yes и statusModerateOperator = Yes
        jooqDbSteps.bannersSteps()
                .addModerateBannerPage(bid, (long) PAGE_ID_6, 0, ModerateBannerPagesStatusmoderate.Yes,
                        ModerateBannerPagesStatusmoderateoperator.Yes, TASK_URL);

        //не отправляем, т.к. statusModerate = No
        jooqDbSteps.bannersSteps()
                .addModerateBannerPage(bid, (long) PAGE_ID_7, 0, ModerateBannerPagesStatusmoderate.No,
                        ModerateBannerPagesStatusmoderateoperator.Yes, TASK_URL);
        //вердикты с statusModerateOperator = None - не отправляем в БК
        jooqDbSteps.bannersSteps()
                .addModerateBannerPage(bid, (long) PAGE_ID_1, ModerateBannerPagesStatusmoderate.Ready);
        jooqDbSteps.bannersSteps()
                .addModerateBannerPageWithUrl(bid, (long) PAGE_ID_2, ModerateBannerPagesStatusmoderate.Yes, TASK_URL);
        jooqDbSteps.bannersSteps()
                .addModerateBannerPageWithUrl(bid, (long) PAGE_ID_3, ModerateBannerPagesStatusmoderate.No, TASK_URL);
        jooqDbSteps.bannersSteps()
                .addModerateBannerPageWithUrl(bid, (long) PAGE_ID_4, ModerateBannerPagesStatusmoderate.Maybe, TASK_URL);
        //удаленная запись, не отправляем в БК
        jooqDbSteps.bannersSteps()
                .addModerateBannerPage(bid, (long) PAGE_ID_5, 1, ModerateBannerPagesStatusmoderate.Yes,
                        ModerateBannerPagesStatusmoderateoperator.Yes, null);
    }

    @Test
    public void testSendingToBs() {
        RunBsTransportScriptResponse resp
                = api.userSteps.getDarkSideSteps().getTransportSteps().sendNewCampaign(shard, cid);
        Campaign campaign =
                api.userSteps.getDarkSideSteps().getTransportSteps().getClientDataRequestCampaign(resp, 0, cid);
        Context context = campaign != null ? campaign.getContext(pid) : null;
        Banner banner = context != null ? context.getBanner(bid) : null;
        assumeThat("в запросе есть outdoor баннер", banner, notNullValue());

        assertThat("параметры  outdoor баннера соответсвуют ожиданиям", banner.getPageModeration(),
                contains(beanDiffer(new PageModeration().withPageId(PAGE_ID_6)
                                .withStatusModerate("Yes").withVersion(1).withTaskUrl(TASK_URL))));
    }
}
