package ru.yandex.autotests.directintapi.bstransport.main.dynbanner.actions.creation.transmit.req1;

import java.util.List;

import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Test;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.utils.tags.TagDictionary;
import ru.yandex.autotests.directapi.darkside.Logins;
import ru.yandex.autotests.directapi.darkside.connection.Semaphore;
import ru.yandex.autotests.directapi.darkside.model.RunBsTransportScriptResponse;
import ru.yandex.autotests.directapi.darkside.model.Status;
import ru.yandex.autotests.directapi.darkside.model.bslogs.StopFlag;
import ru.yandex.autotests.directapi.darkside.model.bslogs.UpdateInfo;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Banner;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Campaign;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Context;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directintapi.bstransport.FeatureNames;
import ru.yandex.autotests.directintapi.bstransport.StoriesNames;
import ru.yandex.autotests.directintapi.bstransport.TransportHelpSteps;
import ru.yandex.autotests.directintapi.bstransport.TransportTestUtils;
import ru.yandex.qatools.Tag;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Stories;
import ru.yandex.qatools.allure.annotations.Title;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

import static org.hamcrest.Matchers.hasSize;
import static ru.yandex.autotests.direct.utils.matchers.BeanEquals.beanEquals;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assertThat;

@Aqua.Test
@Tag(TagDictionary.TRUNK)
@Tag(TagDictionary.RELEASE)
@Title("Транспорт: проверка не-отправки в БК нового остановленного дин. баннера " +
        "в синхронизированной кампании (1 запрос)")
@Stories(StoriesNames.DYN_BANNER_ACTION_CREATION)
@Features({FeatureNames.DYN_BANNERS, FeatureNames.NOT_FOR_FULL_EXPORT})
public class BsTransportOfNewStoppedDynBannerInSyncedCampaignTest {

    private static final String LOGIN = Logins.LOGIN_TRANSPORT;

    @ClassRule
    public static ApiSteps api = new ApiSteps().as(LOGIN);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();


    private static TransportHelpSteps helpSteps = new TransportHelpSteps(api);

    private static int shard;

    private static Long pid;
    private static Long syncedBid;
    private static Long newBid;

    private static Context context;
    private static List<Banner> banners;
    private static Banner syncedBanner;
    private static Banner newBanner;

    @BeforeClass
    public static void beforeClass() {
        shard = api.userSteps.clientFakeSteps().getUserShard(LOGIN);

        Long cid = api.userSteps.campaignSteps().addDefaultDynamicTextCampaign();
        pid = api.userSteps.adGroupsSteps().addDefaultGroupDynamic(cid);
        api.userSteps.dynamicTextAdTargetsSteps().addDefaultWebpage(pid);
        syncedBid = api.userSteps.adsSteps().addDefaultDynamicTextAd(pid);
        newBid = api.userSteps.adsSteps().addDefaultDynamicTextAd(pid);
        Long stoppedBid = api.userSteps.adsSteps().addDefaultDynamicTextAd(pid);

        api.userSteps.campaignFakeSteps().makeNewCampaignReadyForSendingToBS(cid);
        api.userSteps.groupFakeSteps().makeGroupFullyModerated(pid);
        api.userSteps.bannersFakeSteps().makeBannerFullyModerated(syncedBid);
        api.userSteps.bannersFakeSteps().makeBannerFullyModerated(newBid);
        api.userSteps.bannersFakeSteps().makeBannerFullyModerated(stoppedBid);

        // выставляем баннерам статус statusPostModerate = No, чтобы они не отправились
        api.userSteps.getDarkSideSteps().getBannersFakeSteps().setStatusPostModerate(newBid, Status.NO);
        api.userSteps.getDarkSideSteps().getBannersFakeSteps().setStatusPostModerate(stoppedBid, Status.NO);
        api.userSteps.getDarkSideSteps().getTransportSteps().sendNewCampaign(shard, cid);

        // имитируем поведение модерации DIRECT-33440
        api.userSteps.groupFakeSteps().setGroupFakeStatusBsSynced(pid, Status.NO);
        // выставляем баннерам положительный статус модерации
        api.userSteps.getDarkSideSteps().getBannersFakeSteps().setStatusPostModerate(newBid, Status.YES);
        api.userSteps.getDarkSideSteps().getBannersFakeSteps().setStatusPostModerate(stoppedBid, Status.YES);

        api.userSteps.adsSteps().adsSuspend(stoppedBid);

        RunBsTransportScriptResponse resp = api.userSteps.getDarkSideSteps().getTransportSteps().sendNewCampaign(shard, cid);
        Campaign campaign = api.userSteps.getDarkSideSteps().getTransportSteps().getClientDataRequestCampaign(resp, 0, cid);
        context = campaign != null ? campaign.getContext(pid) : null;
        banners = context != null ? TransportTestUtils.getMapValues(context.getBanners()) : null;
        syncedBanner = context != null ? context.getBanner(syncedBid) : null;
        newBanner = context != null ? context.getBanner(newBid) : null;
    }


    @Test
    @Title("Не-отправка в БК нового остановленного дин. баннера в синхронизированной кампании (1 запрос) - " +
            "проверка объекта Контекст")
    public void testTransportOfContextReq1() {
        Context expectedContext = helpSteps.buildExpectedContextObjFromDb(pid, shard, UpdateInfo.UPDATE);
        assertThat("отправленные в БК данные новой группы соответствуют ожидаемым",
                context, beanEquals(expectedContext));
    }

    @Test
    @Title("Не-отправка в БК нового остановленного дин. баннера в синхронизированной кампании (1 запрос) - " +
            "проверка объекта Баннер (синхронизированный)")
    public void testTransportOfSyncedBannerReq1() {
        Banner expectedBanner = helpSteps.buildExpectedBannerObjFromDb(syncedBid, shard, UpdateInfo.SKIP);
        expectedBanner.setStop(StopFlag.RESUME.value());
        assertThat("отправленные в БК данные нового баннера соответствуют ожидаемым",
                syncedBanner, beanEquals(expectedBanner));
    }

    @Test
    @Title("Не-отправка в БК нового остановленного дин. баннера в синхронизированной кампании (1 запрос) - " +
            "проверка объекта Баннер (новый)")
    public void testTransportOfNewBannerReq1() {
        Banner expectedBanner = helpSteps.buildExpectedNewBannerObjFromDb(newBid, shard, UpdateInfo.UPDATE);
        assertThat("отправленные в БК данные нового баннера соответствуют ожидаемым",
                newBanner, beanEquals(expectedBanner));
    }

    @Test
    @Title("Не-отправка в БК нового остановленного дин. баннера в синхронизированной кампании (1 запрос) - " +
            "проверка размера списка баннеров (отсутствие остановленного")
    public void testTransportOfBannerListReq1() {
        assertThat("количество отправленных в БК баннеров соответствует ожидаемому", banners, hasSize(2));
    }
}
