package ru.yandex.autotests.directintapi.bstransport.main.dynbanner.parameters.bannerland;

import java.util.Collection;
import java.util.List;

import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.db.models.jooq.ppc.tables.records.MinusWordsRecord;
import ru.yandex.autotests.direct.db.steps.DirectJooqDbSteps;
import ru.yandex.autotests.direct.utils.tags.TagDictionary;
import ru.yandex.autotests.directapi.darkside.Logins;
import ru.yandex.autotests.directapi.darkside.connection.Semaphore;
import ru.yandex.autotests.directapi.darkside.model.RunBsTransportScriptResponse;
import ru.yandex.autotests.directapi.darkside.model.bslogs.UpdateInfo;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Banner;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.BannerLandDataDynamic;
import ru.yandex.autotests.directapi.model.User;
import ru.yandex.autotests.directapi.model.api5.adgroups.AdGroupUpdateItemMap;
import ru.yandex.autotests.directapi.model.api5.adgroups.UpdateRequestMap;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.autotests.directintapi.bstransport.FeatureNames;
import ru.yandex.autotests.directintapi.bstransport.StoriesNames;
import ru.yandex.autotests.irt.testutils.json.JsonUtils;
import ru.yandex.qatools.Tag;
import ru.yandex.qatools.allure.annotations.Description;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Issue;
import ru.yandex.qatools.allure.annotations.Step;
import ru.yandex.qatools.allure.annotations.Stories;
import ru.yandex.qatools.allure.annotations.Title;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

import static java.util.Arrays.asList;
import static java.util.Collections.singletonList;
import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.not;
import static org.hamcrest.Matchers.isEmptyString;
import static org.hamcrest.Matchers.notNullValue;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assertThat;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assumeThat;

@Aqua.Test
@Tag(TagDictionary.TRUNK)
@Tag(TagDictionary.RELEASE)
@Issue("https://st.yandex-team.ru/DIRECT-90582")
@Title("Транспорт: проверка отправки в БК BannerLandData для дин. баннера c UpdateInfo=1, поле AdGroupMinusWords с библиотечными элементами")
@Stories(StoriesNames.DYN_BANNER_PARAMS_BANNERLAND)
@Features(FeatureNames.DYN_BANNERS)
@RunWith(Parameterized.class)
public class BsTransportOfBannerLandDataAdGroupsLibMinusPhrasesTest {
    private static final String LOGIN = Logins.LOGIN_TRANSPORT;
    private final String FAKE_MINUSWORD = "libminusword2";  //Todo: using it until api will apply minus phrases.

    @ClassRule
    public static ApiSteps api = new ApiSteps().as(LOGIN);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    private static int shard;
    private static Long clientId;

    @Rule
    public Trashman trashman = new Trashman(api);

    @Parameterized.Parameter(0)
    public String[] adGroupMinusWords;

    @Parameterized.Parameter(1)
    public List<String[]> adGroupLibMinusWords;

    @Parameterized.Parameter(2)
    public String expectedAdGroupMinusWords;

    private Banner banner;

    @Parameterized.Parameters(name = "ожидаем в BannerLandData.AdGroupMinusWords={2}")
    public static Collection<Object[]> data() {
        return asList(
                new Object[]{new String[]{"минус фраза"},
                        singletonList(new String[]{"минус фраза"}),
                        "(минус фраза)"
                }, //удаление дублей
                new Object[]{new String[]{"минусслово1", "минус фраза", "минусслово2"},
                        singletonList(new String[]{"минусслово1", "минус фраза", "минусслово2"}),
                        "(минус фраза) минусслово1 минусслово2"
                }, //удаление дублей
                new Object[]{new String[]{"минусслово", "минус фраза"},
                        singletonList(new String[]{"минусслово2", "минус фраза2"}),
                        "(минус фраза) (минус фраза2) минусслово минусслово2"
                },
                new Object[]{new String[]{"минус фраза1", "минус фраза2"},
                        singletonList(new String[]{"минус фраза3", "минус фраза3"}),
                        "(минус фраза1) (минус фраза2) (минус фраза3)"
                },
                new Object[]{new String[]{"минус фраза1", "минус фраза2"},
                        asList(new String[]{"минус фраза3", "минус фраза4"},
                                new String[]{"минус фраза5", "минус фраза6"}),
                        "(минус фраза1) (минус фраза2) (минус фраза3) (минус фраза4) (минус фраза5) (минус фраза6)"
                }
        );
    }

    @BeforeClass
    public static void beforeClass() {
        api.userSteps.clientFakeSteps().fakeClearClientSpentUnits(LOGIN);
        shard = api.userSteps.clientFakeSteps().getUserShard(LOGIN);
        clientId = Long.valueOf(User.get(LOGIN).getClientID());
    }

    @Step("Предусловия: создаем кампанию с минус-фразами на группу")
    @Before
    public void before() {
        DirectJooqDbSteps directJooqDbSteps =
                api.userSteps.getDirectJooqDbSteps().useShard(shard);
        Long cid = api.userSteps.campaignSteps().addDefaultDynamicTextCampaign();
        Long pid = api.userSteps.adGroupsSteps().addDefaultGroupDynamic(cid);
        api.userSteps.dynamicTextAdTargetsSteps().addDefaultWebpage(pid);
        Long bid = api.userSteps.adsSteps().addDefaultDynamicTextAd(pid);

        api.userSteps.campaignFakeSteps().makeNewCampaignReadyForSendingToBS(cid);
        api.userSteps.groupFakeSteps().makeGroupFullyModerated(pid);
        api.userSteps.bannersFakeSteps().makeBannerFullyModerated(bid);

        api.userSteps.adGroupsSteps().adGroupsUpdate(
                new UpdateRequestMap().withAdGroups(
                        new AdGroupUpdateItemMap()
                                .withId(pid)
                                .withNegativeKeywords(FAKE_MINUSWORD)
                )
        );
        MinusWordsRecord mwRecord = directJooqDbSteps.adGroupsSteps().getMinusWordsByPid(pid);
        mwRecord.setMwText(JsonUtils.toString(adGroupMinusWords, true)); //хак: обновляем только текст
        directJooqDbSteps.adGroupsSteps().updateMinusWords(mwRecord);

        adGroupLibMinusWords.forEach(libMw -> {
            Long mwId = directJooqDbSteps.minusWordsSteps().addLibMinusWord(clientId, JsonUtils.toString(libMw, true));
            directJooqDbSteps.adGroupsSteps().addAdGroupMinusWords(pid, mwId);
        });

        RunBsTransportScriptResponse
                resp = api.userSteps.getDarkSideSteps().getTransportSteps().sendNewCampaign(shard, cid);
        banner = api.userSteps.getDarkSideSteps().getTransportSteps().getClientDataRequestCampaign(resp, 0, cid)
                .getContext(pid).getBanner(bid);
    }

    @Test
    @Title("Проверяем поле BannerLandData.AdGroupMinusWords для дин. баннера")
    @Description("Ожидаем, что минус фразы AdGroupMinusWords будут обособленны скобками: (минус фраза)")
    public void testBannerLandDataCampaignMinusWords() {
        assumeThat("в БК отправлен баннер", banner, notNullValue());
        assumeThat("UpdateInfo отправленного баннера " + UpdateInfo.UPDATE
                , banner.getUpdateInfo(), equalTo(UpdateInfo.UPDATE.value()));
        assumeThat("отправленное поле BannerLandData не пустое", banner.getBannerLandData(), not(isEmptyString()));

        String bannerLand = banner.getBannerLandData().replaceAll("\\\\", "");
        BannerLandDataDynamic bannerLandDataDynamic = JsonUtils.getObject(bannerLand, BannerLandDataDynamic.class);

        assertThat("отправленное в БК поле BannerLandData.AdGroupMinusWords соответствуют ожиданиям",
                bannerLandDataDynamic.getAdGroupMinusWords(), equalTo(expectedAdGroupMinusWords));
    }
}
