package ru.yandex.autotests.directintapi.bstransport.main.dynbanner.parameters.bannerland.feed;

import java.util.List;
import java.util.Map;

import one.util.streamex.StreamEx;
import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;

import ru.yandex.autotests.direct.db.models.jooq.ppc.tables.records.GroupParamsRecord;
import ru.yandex.autotests.directapi.darkside.Logins;
import ru.yandex.autotests.directapi.darkside.connection.Semaphore;
import ru.yandex.autotests.directapi.darkside.model.HrefMacrosDTO;
import ru.yandex.autotests.directapi.darkside.model.RunBsTransportScriptResponse;
import ru.yandex.autotests.directapi.darkside.model.Status;
import ru.yandex.autotests.directapi.darkside.model.bslogs.UpdateInfo;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Banner;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.BannerLandDataDynamicFeed;
import ru.yandex.autotests.directapi.model.User;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.autotests.irt.testutils.json.JsonUtils;
import ru.yandex.qatools.allure.annotations.Step;
import ru.yandex.qatools.allure.annotations.Title;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.not;
import static org.hamcrest.Matchers.isEmptyString;
import static org.hamcrest.Matchers.notNullValue;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assertThat;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assumeThat;

/**
 * Created by buhter on 02.08.2016
 * https://st.yandex-team.ru/TESTIRT-9516
 */
@RunWith(Parameterized.class)
public abstract class BsTransportOfFeedDTOBannerLandDataHrefParamsMacrosTestBase {
    static final String PARAMETERIZED = "Значение в Href баннера = {0}, отправляемый макрос = {1}";
    // на сколько тестов дробим. должно совпадать собственно с числом тестов :) и каждый должен использовать свою часть
    // приходится идти на эту хитрость из-за огромного размера логов, вешающих акву
    private static final int BUCKETS_NUM = 6;
    static final Map<Integer, List<Object[]>> CASES = StreamEx.of(HrefMacrosDTO.values())
            .mapToEntry(Enum::ordinal)
            .invert()
            .mapKeys(o -> o % BUCKETS_NUM)
            .mapValues(m -> new Object[]{m.getValue(), m.getMacros()})
            .grouping();

    private static final String LOGIN = Logins.LOGIN_TRANSPORT;

    @ClassRule
    public static ApiSteps api = new ApiSteps().as(LOGIN);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    private static int shard;

    @Rule
    public Trashman trashman = new Trashman(api);

    @Parameterized.Parameter(0)
    public String value;

    @Parameterized.Parameter(1)
    public String macros;

    private String hrefParams;
    private long feedId;
    private Banner banner;

    @BeforeClass
    public static void beforeClass() {
        shard = api.userSteps.clientFakeSteps().getUserShard(LOGIN);
    }

    @Step("Подготовка данных для теста")
    @Before
    public void before() {
        Long cid = api.userSteps.campaignSteps().addDefaultDynamicTextCampaign();
        Long pid = api.userSteps.adGroupsSteps().addDefaultGroupDynamic(cid);
        api.userSteps.dynamicTextAdTargetsSteps().addDefaultWebpage(pid);
        Long bid = api.userSteps.adsSteps().addDefaultDynamicTextAd(pid);
        feedId = api.userSteps.getDirectJooqDbSteps().useShard(shard).feedsSteps()
                .createDefaultFeed(User.get(LOGIN).getClientID());
        api.userSteps.getDirectJooqDbSteps().adGroupsSteps().setAdgroupsDynamicDomainId(pid, null);
        api.userSteps.getDirectJooqDbSteps().adGroupsSteps().setAdgroupsDynamicFeedId(pid, feedId);

        api.userSteps.campaignFakeSteps().makeNewCampaignReadyForSendingToBS(cid);
        api.userSteps.groupFakeSteps().makeGroupFullyModerated(pid);
        api.userSteps.bannersFakeSteps().makeBannerFullyModerated(bid);

        hrefParams = String.format("value={%s}", value);
        api.userSteps.bannersFakeSteps().setStatusModerate(bid, Status.YES);
        api.userSteps.bannersFakeSteps().setStatusPostModerate(bid, Status.YES);
        api.userSteps.groupFakeSteps().makeGroupFullyModerated(pid);
        GroupParamsRecord groupParams = new GroupParamsRecord()
                .setPid(pid)
                .setHasPhraseidHref(1)
                .setHrefParams(hrefParams);
        api.userSteps.getDirectJooqDbSteps().adGroupsSteps().saveGroupParams(groupParams);
        RunBsTransportScriptResponse resp = api.userSteps.getDarkSideSteps().getTransportSteps().sendNewCampaign(shard, cid);
        banner = api.userSteps.getDarkSideSteps().getTransportSteps().getClientDataRequestCampaign(resp, 0, cid).getContext(pid).getBanner(bid);
    }

    @Test
    @Title("Отправка в БК BannerLandData для дин. баннера")
    public void testBannerLandData() {
        assumeThat("в БК отправлен баннер", banner, notNullValue());
        assumeThat("UpdateInfo отправленного баннера " + UpdateInfo.UPDATE
                , banner.getUpdateInfo(), equalTo(UpdateInfo.UPDATE.value()));
        assumeThat("отправленное поле BannerLandData не пустое", banner.getBannerLandData(), not(isEmptyString()));

        String bannerLand = banner.getBannerLandData().replaceAll("\\\\", "");
        BannerLandDataDynamicFeed BannerLandDataDynamicFeed = JsonUtils.getObject(bannerLand, BannerLandDataDynamicFeed.class);
        assertThat("отправленное в БК поле BannerLandData.HrefParams соответствуют ожидаемому",
                BannerLandDataDynamicFeed.getHrefParams(), equalTo(hrefParams.replace(value, macros)));
    }

    @After
    public void clearFeed() {
        if (feedId > 0) {
            api.userSteps.getDirectJooqDbSteps().useShard(shard).feedsSteps().deleteFeedAndAllLinksByFeedId(feedId);
        }
    }
}
