package ru.yandex.autotests.directintapi.bstransport.main.dynimage.creating.transmit.req1;

import java.util.List;

import org.hamcrest.Matcher;
import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Test;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.BannerImagesStatusmoderate;
import ru.yandex.autotests.direct.utils.tags.TagDictionary;
import ru.yandex.autotests.directapi.darkside.Logins;
import ru.yandex.autotests.directapi.darkside.connection.Semaphore;
import ru.yandex.autotests.directapi.darkside.model.ImageType;
import ru.yandex.autotests.directapi.darkside.model.RunBsTransportScriptResponse;
import ru.yandex.autotests.directapi.darkside.model.Status;
import ru.yandex.autotests.directapi.darkside.model.bslogs.StopFlag;
import ru.yandex.autotests.directapi.darkside.model.bslogs.UpdateInfo;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Banner;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Campaign;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Context;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Dynamic;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Images;
import ru.yandex.autotests.directapi.model.api5.ads.AdUpdateItemMap;
import ru.yandex.autotests.directapi.model.api5.ads.DynamicTextAdUpdateMap;
import ru.yandex.autotests.directapi.model.api5.ads.UpdateRequestMap;
import ru.yandex.autotests.directapi.model.api5.general.ExpectedResult;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directintapi.bstransport.FeatureNames;
import ru.yandex.autotests.directintapi.bstransport.StoriesNames;
import ru.yandex.autotests.directintapi.bstransport.TransportHelpSteps;
import ru.yandex.autotests.directintapi.bstransport.TransportTestUtils;
import ru.yandex.autotests.directintapi.bstransport.matchers.images.BannerRegularImagesMatcher;
import ru.yandex.qatools.Tag;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Stories;
import ru.yandex.qatools.allure.annotations.Title;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

import static org.hamcrest.Matchers.empty;
import static org.hamcrest.Matchers.not;
import static ru.yandex.autotests.direct.utils.matchers.BeanEquals.beanEquals;
import static ru.yandex.autotests.directintapi.bstransport.TransportTestUtils.getDoNotSendExtraBannerImageFormatToBsProperty;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assertThat;

@Aqua.Test
@Tag(TagDictionary.TRUNK)
@Tag(TagDictionary.RELEASE)
@Title("Транспорт: проверка отправки в БК новой картинки " +
        "в синхронизированном дин. баннере (1 запрос)")
@Stories(StoriesNames.DYN_IMAGE_ACTION_CREATION)
@Features({FeatureNames.DYN_IMAGES, FeatureNames.NOT_FOR_FULL_EXPORT})
public class BsTransportOfNewPicInSyncedDynBannerTest {
    //DIRECT-49876

    private static final String LOGIN = Logins.LOGIN_TRANSPORT_IMG_REGULAR;

    @ClassRule
    public static ApiSteps api = new ApiSteps().as(LOGIN);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    private static TransportHelpSteps helpSteps = new TransportHelpSteps(api);

    private static int shard;

    private static Long pid;
    private static Long bid;
    private static Long imageId;

    private static Context context;
    private static List<Dynamic> dynamics;
    private static Banner picBanner;
    private static Banner pic;
    private static Images images;

    @BeforeClass
    public static void beforeClass() {
        shard = api.userSteps.clientFakeSteps().getUserShard(LOGIN);
        String[] imageHashes = helpSteps.addImagesForUserIfNotExists(LOGIN, ImageType.REGULAR);

        Long cid = api.userSteps.campaignSteps().addDefaultDynamicTextCampaign();
        pid = api.userSteps.adGroupsSteps().addDefaultGroupDynamic(cid);
        api.userSteps.dynamicTextAdTargetsSteps().addDefaultWebpage(pid);
        bid = api.userSteps.adsSteps().addDefaultDynamicTextAd(pid);

        api.userSteps.campaignFakeSteps().makeNewCampaignReadyForSendingToBS(cid);
        api.userSteps.groupFakeSteps().makeGroupFullyModerated(pid);
        api.userSteps.bannersFakeSteps().makeBannerFullyModerated(bid);

        api.userSteps.getDarkSideSteps().getTransportSteps().sendNewCampaign(shard, cid);
        api.userSteps.adsSteps().shouldGetResultOnUpdate(new UpdateRequestMap().withAds(new AdUpdateItemMap()
                        .withId(bid)
                        .withDynamicTextAd(new DynamicTextAdUpdateMap().withAdImageHash(imageHashes[0]))),
                LOGIN,
                ExpectedResult.success());
        imageId = api.userSteps.getDirectJooqDbSteps().useShard(shard).
                imagesSteps().getBannerImagesByBid(bid).getImageId();
        api.userSteps.getDirectJooqDbSteps().useShard(shard).imagesSteps().setBannerImagesStatusModerate(imageId, BannerImagesStatusmoderate.Yes);
        api.userSteps.groupFakeSteps().setGroupFakeStatusBsSynced(pid, Status.NO);//до починки DIRECT-49882

        RunBsTransportScriptResponse resp = api.userSteps.getDarkSideSteps().getTransportSteps().sendCampaign(shard, cid);
        Campaign campaign = api.userSteps.getDarkSideSteps().getTransportSteps().getClientDataRequestCampaign(resp, 0, cid);
        context = campaign != null ? campaign.getContext(pid) : null;
        dynamics = TransportTestUtils.getMapValues(context != null ? context.getDynamics() : null);
        picBanner = context != null ? context.getBanner(bid) : null;
        pic = context != null ? context.getBanner(imageId) : null;
        images = pic != null ? pic.getImages() : null;
    }

    @Test
    @Title("Отправка в БК новой картинки в синхронизированном дин. баннере (1 запрос) - " +
            "проверка объекта Контекст")
    public void testTransportOfContextObjReq1() {
        Context expectedContext =
                helpSteps.buildExpectedContextObjFromDb(pid, shard, UpdateInfo.UPDATE);
        assertThat("отправленные в БК данные группы соответствуют ожидаемым",
                context, beanEquals(expectedContext));
    }

    @Test
    @Title("Отправка в БК новой картинки в синхронизированном дин. баннере (1 запрос) - " +
            "проверка списка Нацеливаний")
    public void testTransportOfDynamicsListReq1() {
        assertThat("в БК отправлен не пустой список фраз", dynamics, not(empty()));
    }

    @Test
    @Title("Отправка в БК новой картинки в синхронизированном дин. баннере (1 запрос) - " +
            "проверка объекта Баннер")
    public void testTransportOfNewBannerObjReq1() {
        Banner expectedPicBanner = helpSteps.buildExpectedBannerObjFromDb(bid, shard, UpdateInfo.UPDATE);
        expectedPicBanner.setStop(StopFlag.RESUME.value());
        assertThat("отправленные в БК данные нового баннера соответствуют ожидаемым",
                picBanner, beanEquals(expectedPicBanner));
    }

    @Test
    @Title("Отправка в БК новой картинки в синхронизированном дин. баннере (1 запрос) - " +
            "проверка объекта Картинка")
    public void testTransportOfNewPicObjReq1() {
        Matcher<Banner> picMatcher =
                helpSteps.buildNewPicBannerMatcherFromDbWithBannerId(imageId, shard, UpdateInfo.UPDATE);
        assertThat("отправленные в БК данные новой картинки соответствуют ожидаемым", pic, picMatcher);
    }

    @Test
    @Title("Отправка в БК новой картинки в синхронизированном дин. баннере (1 запрос) - " +
            "проверка форматов картинок")
    public void testTransportOfImagesObjReq1() {
        Matcher<Images> imagesMatcher = BannerRegularImagesMatcher.isValid(getDoNotSendExtraBannerImageFormatToBsProperty(api));
        assertThat("отправленные в БК форматы картинок соответствуют ожидаемым", images, imagesMatcher);
    }
}
