package ru.yandex.autotests.directintapi.bstransport.main.group.minus_geo;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;

import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.BannersMinusGeoType;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.BannersStatusbssynced;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.BannersStatuspostmoderate;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.BannersStatusshow;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.PhrasesStatusbssynced;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.PhrasesStatuspostmoderate;
import ru.yandex.autotests.direct.db.models.jooq.ppc.tables.records.BannersMinusGeoRecord;
import ru.yandex.autotests.direct.db.models.jooq.ppc.tables.records.BannersRecord;
import ru.yandex.autotests.direct.db.models.jooq.ppc.tables.records.PhrasesRecord;
import ru.yandex.autotests.direct.db.steps.DirectJooqDbSteps;
import ru.yandex.autotests.direct.utils.tags.TagDictionary;
import ru.yandex.autotests.directapi.darkside.Logins;
import ru.yandex.autotests.directapi.darkside.connection.Semaphore;
import ru.yandex.autotests.directapi.darkside.model.RunBsTransportScriptResponse;
import ru.yandex.autotests.directapi.darkside.model.bslogs.StopFlag;
import ru.yandex.autotests.directapi.darkside.model.bslogs.UpdateInfo;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Banner;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Campaign;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Context;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directintapi.bstransport.FeatureNames;
import ru.yandex.autotests.directintapi.bstransport.StoriesNames;
import ru.yandex.autotests.directintapi.bstransport.TestRegionsSet;
import ru.yandex.qatools.Tag;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Issue;
import ru.yandex.qatools.allure.annotations.Step;
import ru.yandex.qatools.allure.annotations.Stories;
import ru.yandex.qatools.allure.annotations.Title;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

import static org.hamcrest.Matchers.containsInAnyOrder;
import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.notNullValue;
import static org.hamcrest.Matchers.nullValue;
import static org.junit.Assert.assertThat;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assumeThat;

/**
 * Created by n-boy on 03.03.17.
 * https://st.yandex-team.ru/TESTIRT-11106
 */

@Aqua.Test
@Tag(TagDictionary.RELEASE)
@Issue("https://st.yandex-team.ru/DIRECT-63267")
@Title("Транспорт: применение минус-регионов, для одиночного, побывавшего в БК баннера")
@Stories(StoriesNames.MINUS_GEO)
@Features(FeatureNames.GROUPS)
@RunWith(Parameterized.class)
public class BsTransportOfMinusGeoForTextOldBannerTest {
    private static final String login = Logins.LOGIN_TRANSPORT;
    private static int shard;
    private static Long cid;
    private static Long pid;
    private static Long bid;
    private static Context context;
    private static Banner banner;

    private static String defaultAdgroupSrcGeo = TestRegionsSet.defaultAdgroupSrcGeo;
    private static String defaultBannerCurentMinusGeo = TestRegionsSet.defaultBannerCurentMinusGeo;
    private static String defaultBannerBsSyncedMinusGeo = TestRegionsSet.defaultBannerBsSyncedMinusGeo;
    private static String adgroupGeoExceptCurrent = TestRegionsSet.adgroupGeoExceptCurrent;
    private static String adgroupGeoExceptBsSynced = TestRegionsSet.adgroupGeoExceptBsSynced;

    private static String wholeWorldGeo = TestRegionsSet.wholeWorldGeo;
    private static String wholeWorldGeoExceptCurrent = TestRegionsSet.wholeWorldGeoExceptCurrent;

    private static String noWhereGeo = TestRegionsSet.noWhereGeo;

    private static DirectJooqDbSteps dbSteps;

    private static HashMap<Integer, Boolean> testDataPreparedWithParams = new HashMap<Integer, Boolean>();

    @ClassRule
    public static ApiSteps api = new ApiSteps().as(login);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    @Parameterized.Parameter(0)
    public int testCaseNumber;

    @Parameterized.Parameter(1)
    public PhrasesStatuspostmoderate adgroupStatusPostModerate;

    @Parameterized.Parameter(2)
    public PhrasesStatusbssynced adgroupStatusBsSynced;

    @Parameterized.Parameter(3)
    public BannersStatuspostmoderate bannerStatusPostModerate;

    @Parameterized.Parameter(4)
    public BannersStatusshow bannerStatusShow;

    @Parameterized.Parameter(5)
    public BannersStatusbssynced bannerStatusBsSynced;

    @Parameterized.Parameter(6)
    public String adgroupSrcGeo;

    @Parameterized.Parameter(7)
    public String bannerCurentMinusGeo;

    @Parameterized.Parameter(8)
    public String bannerBsSyncedMinusGeo;

    @Parameterized.Parameter(9)
    public UpdateInfo resContextUpdateInfo;

    @Parameterized.Parameter(10)
    public UpdateInfo resBannerUpdateInfo;

    @Parameterized.Parameter(11)
    public StopFlag resBannerStop;

    @Parameterized.Parameter(12)
    public String resContextGeo;

    @Parameterized.Parameter(13)
    public String resBannerBsSyncedMinusGeo;

    @Parameterized.Parameters(name = "Номер тест-кейса: {0}, свойства группы/баннера: "
            + "adgroupStatusPostModerate={1}, "
            + "adgroupStatusBsSynced={2}, "
            + "bannerStatusPostModerate={3}, "
            + "bannerStatusShow={4}, "
            + "bannerStatusBsSynced={5}, "
            + "adgroupGeo={6}, "
            + "bannerCurentMinusGeo={7}, "
            + "bannerBsSyncedMinusGeo={8}")
    public static Collection<Object[]> testData() {
        return Arrays.asList(
                new Object[]{2, PhrasesStatuspostmoderate.Yes, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Yes, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.UPDATE, UpdateInfo.UPDATE, StopFlag.RESUME, adgroupGeoExceptCurrent, defaultBannerCurentMinusGeo},
                new Object[]{3, PhrasesStatuspostmoderate.Rejected, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Yes, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.UPDATE, UpdateInfo.SKIP, StopFlag.STOP, defaultAdgroupSrcGeo, defaultBannerBsSyncedMinusGeo},
                new Object[]{4, PhrasesStatuspostmoderate.Rejected, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Yes, BannersStatusshow.No, BannersStatusbssynced.No, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.UPDATE, UpdateInfo.SKIP, StopFlag.STOP, defaultAdgroupSrcGeo, defaultBannerBsSyncedMinusGeo},
                new Object[]{5, PhrasesStatuspostmoderate.No, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Yes, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, null, null, null, null, null, null},
                new Object[]{6, PhrasesStatuspostmoderate.No, PhrasesStatusbssynced.Yes, BannersStatuspostmoderate.Yes, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, null, null, null, null, null, null},
                new Object[]{7, PhrasesStatuspostmoderate.Rejected, PhrasesStatusbssynced.Yes, BannersStatuspostmoderate.Yes, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, null, UpdateInfo.SKIP, UpdateInfo.SKIP, StopFlag.STOP, null, null},
                new Object[]{8, PhrasesStatuspostmoderate.No, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Yes, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, null, defaultBannerBsSyncedMinusGeo, null, null, null, null, defaultBannerBsSyncedMinusGeo},
                new Object[]{9, PhrasesStatuspostmoderate.No, PhrasesStatusbssynced.Yes, BannersStatuspostmoderate.Yes, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, null, defaultBannerBsSyncedMinusGeo, null, null, null, null, defaultBannerBsSyncedMinusGeo},
                new Object[]{10, PhrasesStatuspostmoderate.Rejected, PhrasesStatusbssynced.Yes, BannersStatuspostmoderate.Yes, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, null, defaultBannerBsSyncedMinusGeo, UpdateInfo.SKIP, UpdateInfo.SKIP, StopFlag.STOP, null, defaultBannerBsSyncedMinusGeo},
                new Object[]{11, PhrasesStatuspostmoderate.No, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Yes, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, null, null, UpdateInfo.SKIP, UpdateInfo.UPDATE, StopFlag.RESUME, null, null},
                new Object[]{12, PhrasesStatuspostmoderate.No, PhrasesStatusbssynced.Yes, BannersStatuspostmoderate.Yes, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, null, null, UpdateInfo.SKIP, UpdateInfo.UPDATE, StopFlag.RESUME, null, null},
                new Object[]{13, PhrasesStatuspostmoderate.Rejected, PhrasesStatusbssynced.Yes, BannersStatuspostmoderate.Yes, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, null, null, UpdateInfo.SKIP, UpdateInfo.SKIP, StopFlag.STOP, null, null},
                new Object[]{14, PhrasesStatuspostmoderate.Yes, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Rejected, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.UPDATE, UpdateInfo.SKIP, StopFlag.STOP, defaultAdgroupSrcGeo, defaultBannerBsSyncedMinusGeo},
                new Object[]{15, PhrasesStatuspostmoderate.Yes, PhrasesStatusbssynced.No, BannersStatuspostmoderate.No, BannersStatusshow.Yes, BannersStatusbssynced.Yes, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.UPDATE, UpdateInfo.SKIP, StopFlag.RESUME, adgroupGeoExceptBsSynced, defaultBannerBsSyncedMinusGeo},
                new Object[]{16, PhrasesStatuspostmoderate.Yes, PhrasesStatusbssynced.No, BannersStatuspostmoderate.No, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.UPDATE, UpdateInfo.SKIP, StopFlag.RESUME, adgroupGeoExceptBsSynced, defaultBannerBsSyncedMinusGeo},
                new Object[]{17, PhrasesStatuspostmoderate.Rejected, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Rejected, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.UPDATE, UpdateInfo.SKIP, StopFlag.STOP, defaultAdgroupSrcGeo, defaultBannerBsSyncedMinusGeo},
                new Object[]{18, PhrasesStatuspostmoderate.Rejected, PhrasesStatusbssynced.No, BannersStatuspostmoderate.No, BannersStatusshow.Yes, BannersStatusbssynced.Yes, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.UPDATE, UpdateInfo.SKIP, StopFlag.STOP, defaultAdgroupSrcGeo, defaultBannerBsSyncedMinusGeo},
                new Object[]{19, PhrasesStatuspostmoderate.Rejected, PhrasesStatusbssynced.No, BannersStatuspostmoderate.No, BannersStatusshow.No, BannersStatusbssynced.Yes, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.UPDATE, UpdateInfo.SKIP, StopFlag.STOP, defaultAdgroupSrcGeo, defaultBannerBsSyncedMinusGeo},
                new Object[]{20, PhrasesStatuspostmoderate.No, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Rejected, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.SKIP, UpdateInfo.SKIP, StopFlag.STOP, null, defaultBannerBsSyncedMinusGeo},
                new Object[]{22, PhrasesStatuspostmoderate.Yes, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Yes, BannersStatusshow.No, BannersStatusbssynced.No, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.UPDATE, UpdateInfo.SKIP, StopFlag.STOP, defaultAdgroupSrcGeo, defaultBannerBsSyncedMinusGeo},
                new Object[]{23, PhrasesStatuspostmoderate.No, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Yes, BannersStatusshow.No, BannersStatusbssynced.No, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.SKIP, UpdateInfo.SKIP, StopFlag.STOP, null, defaultBannerBsSyncedMinusGeo},
                new Object[]{24, PhrasesStatuspostmoderate.Yes, PhrasesStatusbssynced.Yes, BannersStatuspostmoderate.Yes, BannersStatusshow.No, BannersStatusbssynced.No, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.SKIP, UpdateInfo.SKIP, StopFlag.STOP, null, defaultBannerBsSyncedMinusGeo},
                new Object[]{25, PhrasesStatuspostmoderate.Yes, PhrasesStatusbssynced.Yes, BannersStatuspostmoderate.Rejected, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.SKIP, UpdateInfo.SKIP, StopFlag.STOP, null, defaultBannerBsSyncedMinusGeo},
                new Object[]{27, PhrasesStatuspostmoderate.Yes, PhrasesStatusbssynced.Yes, BannersStatuspostmoderate.Yes, BannersStatusshow.No, BannersStatusbssynced.No, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.SKIP, UpdateInfo.SKIP, StopFlag.STOP, null, defaultBannerBsSyncedMinusGeo},
                new Object[]{28, PhrasesStatuspostmoderate.Yes, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Yes, BannersStatusshow.Yes, BannersStatusbssynced.Yes, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.UPDATE, UpdateInfo.SKIP, StopFlag.RESUME, adgroupGeoExceptBsSynced, defaultBannerBsSyncedMinusGeo},
                new Object[]{29, PhrasesStatuspostmoderate.Yes, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Yes, BannersStatusshow.No, BannersStatusbssynced.Yes, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.UPDATE, UpdateInfo.SKIP, StopFlag.STOP, defaultAdgroupSrcGeo, defaultBannerBsSyncedMinusGeo},
                new Object[]{30, PhrasesStatuspostmoderate.Yes, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Rejected, BannersStatusshow.Yes, BannersStatusbssynced.Yes, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.UPDATE, UpdateInfo.SKIP, StopFlag.STOP, defaultAdgroupSrcGeo, defaultBannerBsSyncedMinusGeo},
                new Object[]{31, PhrasesStatuspostmoderate.Rejected, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Yes, BannersStatusshow.Yes, BannersStatusbssynced.Yes, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.UPDATE, UpdateInfo.SKIP, StopFlag.STOP, defaultAdgroupSrcGeo, defaultBannerBsSyncedMinusGeo},
                new Object[]{32, PhrasesStatuspostmoderate.Rejected, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Yes, BannersStatusshow.No, BannersStatusbssynced.Yes, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.UPDATE, UpdateInfo.SKIP, StopFlag.STOP, defaultAdgroupSrcGeo, defaultBannerBsSyncedMinusGeo},
                new Object[]{35, PhrasesStatuspostmoderate.Yes, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Yes, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, defaultBannerCurentMinusGeo, null, UpdateInfo.UPDATE, UpdateInfo.UPDATE, StopFlag.RESUME, adgroupGeoExceptCurrent, defaultBannerCurentMinusGeo},
                new Object[]{36, PhrasesStatuspostmoderate.Yes, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Yes, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, null, defaultBannerBsSyncedMinusGeo, UpdateInfo.UPDATE, UpdateInfo.UPDATE, StopFlag.RESUME, defaultAdgroupSrcGeo, null},
                new Object[]{37, PhrasesStatuspostmoderate.Yes, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Yes, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, defaultAdgroupSrcGeo, null, UpdateInfo.UPDATE, UpdateInfo.UPDATE, StopFlag.RESUME, noWhereGeo, defaultAdgroupSrcGeo},
                new Object[]{38, PhrasesStatuspostmoderate.Yes, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Yes, BannersStatusshow.Yes, BannersStatusbssynced.No, wholeWorldGeo, defaultBannerCurentMinusGeo, defaultBannerBsSyncedMinusGeo, UpdateInfo.UPDATE, UpdateInfo.UPDATE, StopFlag.RESUME, wholeWorldGeoExceptCurrent, defaultBannerCurentMinusGeo},
                new Object[]{75, PhrasesStatuspostmoderate.Yes, PhrasesStatusbssynced.No, BannersStatuspostmoderate.Yes, BannersStatusshow.Yes, BannersStatusbssynced.No, defaultAdgroupSrcGeo, null, defaultAdgroupSrcGeo, UpdateInfo.UPDATE, UpdateInfo.UPDATE, StopFlag.RESUME, defaultAdgroupSrcGeo, null}
        );
    }

    @BeforeClass
    @Step("Создаем новую кампанию, с группой и баннером, полностью модерируем и отправляем в БК")
    public static void beforeClass() {
        shard = api.userSteps.clientFakeSteps().getUserShard(login);
        dbSteps =  api.userSteps.getDirectJooqDbSteps().useShard(shard);

        cid = api.userSteps.campaignSteps().addDefaultTextCampaign();

        api.userSteps.campaignFakeSteps().makeNewCampaignReadyForSendingToBS(cid);

        pid = api.userSteps.adGroupsSteps().addDefaultGroup(cid);
        bid = api.userSteps.adsSteps().addDefaultTextAd(pid);
        Long keywordId = api.userSteps.keywordsSteps().addDefaultKeyword(pid);

        api.userSteps.groupFakeSteps().makeGroupFullyModerated(pid);
        api.userSteps.phrasesFakeSteps().makeKeywordModerated(keywordId);
        api.userSteps.bannersFakeSteps().makeBannerFullyModerated(bid);

        RunBsTransportScriptResponse
                resp = api.userSteps.getDarkSideSteps().getTransportSteps().sendNewCampaign(shard, cid);
        Campaign campaign = api.userSteps.getDarkSideSteps().getTransportSteps().getClientDataRequestCampaign(resp, 0, cid);
        assumeThat("новая кампания отправлена в БК", campaign, notNullValue());
        context = campaign != null ? campaign.getContext(pid) : null;
        banner = context != null ? context.getBanner(bid) : null;
        assumeThat("в БК отправлен баннер, первой итерацией", banner, notNullValue());
    }

    @Before
    @Step("Применяем к созданным ранее группе/баннеру свойства из параметров теста, и запускаем итерацию транспорта")
    public void prepareTestData() {
        if (testDataPreparedWithParams.getOrDefault(testCaseNumber, false)) {
            return;
        }

        PhrasesRecord phrasesRecord = dbSteps.adGroupsSteps().getPhrases(pid);
        phrasesRecord.setStatuspostmoderate(adgroupStatusPostModerate);
        phrasesRecord.setStatusbssynced(adgroupStatusBsSynced);
        phrasesRecord.setGeo(adgroupSrcGeo);
        dbSteps.adGroupsSteps().updatePhrases(phrasesRecord);

        BannersRecord bannersRecord = dbSteps.bannersSteps().getBanner(bid);
        bannersRecord.setStatuspostmoderate(bannerStatusPostModerate);
        bannersRecord.setStatusbssynced(bannerStatusBsSynced);
        bannersRecord.setStatusshow(bannerStatusShow);
        dbSteps.bannersSteps().updateBanners(bannersRecord);

        if (bannerCurentMinusGeo == null) {
            dbSteps.bannersSteps().deleteBannersMinusGeo(bid, BannersMinusGeoType.current);
        } else {
            dbSteps.bannersSteps().saveBannersMinusGeo(bid, BannersMinusGeoType.current, bannerCurentMinusGeo);
        }
        if (bannerBsSyncedMinusGeo == null) {
            dbSteps.bannersSteps().deleteBannersMinusGeo(bid, BannersMinusGeoType.bs_synced);
        } else {
            dbSteps.bannersSteps().saveBannersMinusGeo(bid, BannersMinusGeoType.bs_synced, bannerBsSyncedMinusGeo);
        }

        api.userSteps.campaignFakeSteps().setStatusBsSynced(cid, "No");
        RunBsTransportScriptResponse resp = api.userSteps.getDarkSideSteps().getTransportSteps().sendSyncedCampaign(shard, cid);
        Campaign campaign = api.userSteps.getDarkSideSteps().getTransportSteps().getClientDataRequestCampaign(resp, cid);
        context = campaign != null ? campaign.getContext(pid) : null;
        banner = context != null ? context.getBanner(bid) : null;

        testDataPreparedWithParams.put(testCaseNumber, true);
    }

    @Test
    @Title("Отправка в БК группы баннеров с учетом минус-регионов, проверяем Context.UpdateInfo")
    public void testContextUpdateInfo() {
        if (resContextUpdateInfo == null) {
            assertThat("условия показа не отправлены в БК", context, nullValue());
        } else {
            assertThat("условия показа отправлены в БК", context, notNullValue());
            assertThat("условия показа отправлены в БК с соответствующим значением UpdateInfo",
                    context.getUpdateInfo(), equalTo(resContextUpdateInfo.value()));
        }
    }

    @Test
    @Title("Отправка в БК баннера с учетом минус-регионов, проверяем Banner.UpdateInfo")
    public void testBannerUpdateInfo() {
        if (resBannerUpdateInfo == null) {
            assertThat("баннер не отправлен в БК", banner, nullValue());
        } else {
            assumeThat("баннер отправлен в БК", banner, notNullValue());
            assertThat("баннер отправлен в БК с соответствующим значением UpdateInfo",
                    banner.getUpdateInfo(), equalTo(resBannerUpdateInfo.value()));
        }
    }

    @Test
    @Title("Отправка в БК баннера с учетом минус-регионов, проверяем Banner.Stop")
    public void testBannerStop() {
        if (resBannerStop != null) {
            assumeThat("баннер отправлен в БК", banner, notNullValue());
            assertThat("баннер отправлен в БК с соответствующим значением Stop",
                    banner.getStop(), equalTo(resBannerStop.value()));
        }
    }

    @Test
    @Title("Отправка в БК группы баннеров с учетом минус-регионов, проверяем Context.Geo")
    public void testContextGeo() {
        if (resContextGeo != null) {
            assumeThat("условия показа отправлены в БК", context, notNullValue());
            String[] resContextGeoList;
            if (resContextGeo.equals("")) {
                resContextGeoList = new String[]{};
            } else {
                resContextGeoList = resContextGeo.split(",");
            }
            assertThat("условия показа отправлены в БК с соответствующим значением Geo",
                    context.getGeo(), containsInAnyOrder(resContextGeoList));
        }
    }

    @Test
    @Title("Отправка в БК баннера с учетом минус-регионов, проверяем состояние записи в БД banners_minus_geo[type=bs_synced]")
    public void testBannerBsSyncedMinusGeo() {
        BannersMinusGeoRecord minusGeo = dbSteps.bannersSteps().getBannersMinusGeo(bid, BannersMinusGeoType.bs_synced);
        if (resBannerBsSyncedMinusGeo == null) {
            assertThat("баннер после отправки в БК, в БД находится с соответствующими минус-регионами типа bs_synced",
                    minusGeo, nullValue());
        } else {
            assertThat("баннер после отправки в БК, в БД находится с соответствующими минус-регионами типа bs_synced",
                    minusGeo == null ? null : minusGeo.getMinusGeo(), equalTo(resBannerBsSyncedMinusGeo));
        }
    }

}
