package ru.yandex.autotests.directintapi.bstransport.main.imagead.transmit;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;

import org.hamcrest.core.IsEqual;
import org.hamcrest.core.IsNull;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.BannerImagesFormatsImageType;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.BannersMinusGeoType;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.BannersStatusbssynced;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.BannersStatuspostmoderate;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.ImagesStatusmoderate;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.PhrasesStatusbssynced;
import ru.yandex.autotests.direct.db.models.jooq.ppc.tables.records.BannersMinusGeoRecord;
import ru.yandex.autotests.direct.db.models.jooq.ppc.tables.records.BannersRecord;
import ru.yandex.autotests.direct.db.models.jooq.ppc.tables.records.PhrasesRecord;
import ru.yandex.autotests.direct.db.steps.DirectJooqDbSteps;
import ru.yandex.autotests.direct.utils.tags.TagDictionary;
import ru.yandex.autotests.directapi.darkside.Logins;
import ru.yandex.autotests.directapi.darkside.connection.Semaphore;
import ru.yandex.autotests.directapi.darkside.model.RunBsTransportScriptResponse;
import ru.yandex.autotests.directapi.darkside.model.bslogs.StopFlag;
import ru.yandex.autotests.directapi.darkside.model.bslogs.UpdateInfo;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Banner;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Campaign;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Context;
import ru.yandex.autotests.directapi.enums.AdGroupType;
import ru.yandex.autotests.directapi.enums.BannerType;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directintapi.bstransport.FeatureNames;
import ru.yandex.autotests.directintapi.bstransport.StoriesNames;
import ru.yandex.autotests.directintapi.bstransport.TestRegionsSet;
import ru.yandex.qatools.Tag;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Issue;
import ru.yandex.qatools.allure.annotations.Step;
import ru.yandex.qatools.allure.annotations.Stories;
import ru.yandex.qatools.allure.annotations.Title;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

import static org.hamcrest.Matchers.containsInAnyOrder;
import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.notNullValue;
import static org.hamcrest.Matchers.nullValue;
import static org.junit.Assert.assertThat;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assumeThat;

/**
 * Created by n-boy on 09.03.17.
 * https://st.yandex-team.ru/TESTIRT-11106
 */

@Aqua.Test
@Tag(TagDictionary.RELEASE)
@Issue("https://st.yandex-team.ru/DIRECT-63267")
@Title("Транспорт: применение минус-регионов, для ГО-баннера")
@Stories(StoriesNames.MINUS_GEO)
@Features(FeatureNames.IMAGE_ADS)
@RunWith(Parameterized.class)
public class BsTransportImageAdWithMinusGeoTest {
    private static final String login = Logins.LOGIN_TRANSPORT_IMG_AD;
    private static int shard;
    private static Long cid;
    private static Long pid;
    private static Long bid;
    private static Context context;
    private static Banner banner;
    private static String imageHash;

    private static String defaultAdgroupSrcGeo = TestRegionsSet.defaultAdgroupSrcGeo;
    private static String defaultBannerCurentMinusGeo = TestRegionsSet.defaultBannerCurentMinusGeo;
    private static String defaultBannerBsSyncedMinusGeo = TestRegionsSet.defaultBannerBsSyncedMinusGeo;
    private static String adgroupGeoExceptCurrent = TestRegionsSet.adgroupGeoExceptCurrent;
    private static String adgroupGeoExceptBsSynced = TestRegionsSet.adgroupGeoExceptBsSynced;

    private static DirectJooqDbSteps dbSteps;

    private static HashMap<Integer, Boolean> testDataPreparedWithParams = new HashMap<Integer, Boolean>();

    @ClassRule
    public static ApiSteps api = new ApiSteps().as(login);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    @Parameterized.Parameter(0)
    public int testCaseNumber;

    @Parameterized.Parameter(1)
    public BannersStatuspostmoderate bannerStatusPostModerate;

    @Parameterized.Parameter(2)
    public ImagesStatusmoderate imageStatusModerate;

    @Parameterized.Parameter(3)
    public UpdateInfo resContextUpdateInfo;

    @Parameterized.Parameter(4)
    public UpdateInfo resBannerUpdateInfo;

    @Parameterized.Parameter(5)
    public StopFlag resBannerStop;

    @Parameterized.Parameter(6)
    public String resContextGeo;

    @Parameterized.Parameter(7)
    public String resBannerBsSyncedMinusGeo;

    @Parameterized.Parameters(name = "Номер тест-кейса: {0}, свойства группы/баннера: "
            + "bannerStatusPostModerate={1}, "
            + "imageStatusModerate={2}" )
    public static Collection<Object[]> testData() {
        return Arrays.asList(
                new Object[]{42, BannersStatuspostmoderate.Yes, ImagesStatusmoderate.Yes, UpdateInfo.UPDATE, UpdateInfo.UPDATE, StopFlag.RESUME, adgroupGeoExceptCurrent, defaultBannerCurentMinusGeo},
                new Object[]{43, BannersStatuspostmoderate.No, ImagesStatusmoderate.Yes, UpdateInfo.UPDATE, UpdateInfo.SKIP, StopFlag.RESUME, adgroupGeoExceptBsSynced, defaultBannerBsSyncedMinusGeo},
                new Object[]{44, BannersStatuspostmoderate.Yes, ImagesStatusmoderate.No, UpdateInfo.UPDATE, UpdateInfo.SKIP, StopFlag.STOP, defaultAdgroupSrcGeo, defaultBannerBsSyncedMinusGeo}
        );
    }

    @BeforeClass
    @Step("Создаем новую кампанию, с группой и ГО-баннером, полностью модерируем и отправляем в БК")
    public static void beforeClass() {
        shard = api.userSteps.clientFakeSteps().getUserShard(login);
        dbSteps =  api.userSteps.getDirectJooqDbSteps().useShard(shard);
        imageHash = api.userSteps.imagesStepsV5().adImagesGetAllHashes(login).get(0);

        cid = api.userSteps.campaignSteps().addDefaultTextCampaign();
        pid = api.userSteps.adGroupsSteps().addDefaultGroup(cid);
        assumeThat("выбрана картинка правильного формата",
                        dbSteps.imagesSteps().getBannerImagesFormatsRecords(imageHash).getImageType(),
                IsEqual.equalTo(BannerImagesFormatsImageType.image_ad));
        bid = api.userSteps.adsSteps().addAdWithAdImageHash(pid, AdGroupType.BASE, BannerType.IMAGE_AD, imageHash);
        Long keywordId = api.userSteps.keywordsSteps().addDefaultKeyword(pid);

        api.userSteps.campaignFakeSteps().makeNewCampaignReadyForSendingToBS(cid);
        api.userSteps.groupFakeSteps().makeGroupFullyModerated(pid);
        api.userSteps.bannersFakeSteps().makeBannerFullyModerated(bid);
        api.userSteps.phrasesFakeSteps().makeKeywordModerated(keywordId);
        dbSteps.imagesSteps().setImageStatusModerate(bid, imageHash, ImagesStatusmoderate.Yes);
        RunBsTransportScriptResponse resp
                = api.userSteps.getDarkSideSteps().getTransportSteps().sendNewCampaign(shard, cid);
        Campaign campaign = api.userSteps.getDarkSideSteps().getTransportSteps().getClientDataRequestCampaign(resp, 0, cid);
        Context context = campaign != null ? campaign.getContext(pid) : null;
        banner = context != null ? context.getBanner(bid) : null;
        assumeThat("в БК отправлен Banner графического баннера, первой итерацией", banner, IsNull.notNullValue());
    }

    @Before
    @Step("Применяем к созданным ранее группе/баннеру свойства из параметров теста, и запускаем итерацию транспорта")
    public void prepareTestData() {
        if (testDataPreparedWithParams.getOrDefault(testCaseNumber, false)) {
            return;
        }

        PhrasesRecord phrasesRecord = dbSteps.adGroupsSteps().getPhrases(pid);
        phrasesRecord.setStatusbssynced(PhrasesStatusbssynced.No);
        phrasesRecord.setGeo(defaultAdgroupSrcGeo);
        dbSteps.adGroupsSteps().updatePhrases(phrasesRecord);

        BannersRecord bannersRecord = dbSteps.bannersSteps().getBanner(bid);
        bannersRecord.setStatuspostmoderate(bannerStatusPostModerate);
        bannersRecord.setStatusbssynced(BannersStatusbssynced.No);
        dbSteps.bannersSteps().updateBanners(bannersRecord);

        dbSteps.imagesSteps().setImageStatusModerate(bid, imageHash, imageStatusModerate);

        dbSteps.bannersSteps().saveBannersMinusGeo(bid, BannersMinusGeoType.current, defaultBannerCurentMinusGeo);
        dbSteps.bannersSteps().saveBannersMinusGeo(bid, BannersMinusGeoType.bs_synced, defaultBannerBsSyncedMinusGeo);

        api.userSteps.campaignFakeSteps().setStatusBsSynced(cid, "No");
        RunBsTransportScriptResponse resp = api.userSteps.getDarkSideSteps().getTransportSteps().sendSyncedCampaign(shard, cid);
        Campaign campaign = api.userSteps.getDarkSideSteps().getTransportSteps().getClientDataRequestCampaign(resp, cid);
        context = campaign != null ? campaign.getContext(pid) : null;
        banner = context != null ? context.getBanner(bid) : null;

        testDataPreparedWithParams.put(testCaseNumber, true);
    }

    @Test
    @Title("Отправка в БК группы баннеров с учетом минус-регионов, проверяем Context.UpdateInfo")
    public void testContextUpdateInfo() {
        if (resContextUpdateInfo == null) {
            assertThat("условия показа не отправлены в БК", context, nullValue());
        } else {
            assertThat("условия показа отправлены в БК", context, notNullValue());
            assertThat("условия показа отправлены в БК с соответствующим значением UpdateInfo",
                    context.getUpdateInfo(), equalTo(resContextUpdateInfo.value()));
        }
    }

    @Test
    @Title("Отправка в БК баннера с учетом минус-регионов, проверяем Banner.UpdateInfo")
    public void testBannerUpdateInfo() {
        if (resBannerUpdateInfo == null) {
            assertThat("баннер не отправлен в БК", banner, nullValue());
        } else {
            assumeThat("баннер отправлен в БК", banner, notNullValue());
            assertThat("баннер отправлен в БК с соответствующим значением UpdateInfo",
                    banner.getUpdateInfo(), equalTo(resBannerUpdateInfo.value()));
        }
    }

    @Test
    @Title("Отправка в БК баннера с учетом минус-регионов, проверяем Banner.Stop")
    public void testBannerStop() {
        if (resBannerStop != null) {
            assumeThat("баннер отправлен в БК", banner, notNullValue());
            assertThat("баннер отправлен в БК с соответствующим значением Stop",
                    banner.getStop(), equalTo(resBannerStop.value()));
        }
    }

    @Test
    @Title("Отправка в БК группы баннеров с учетом минус-регионов, проверяем Context.Geo")
    public void testContextGeo() {
        if (resContextGeo != null) {
            assumeThat("условия показа отправлены в БК", context, notNullValue());
            String[] resContextGeoList;
            if (resContextGeo.equals("")) {
                resContextGeoList = new String[]{};
            } else {
                resContextGeoList = resContextGeo.split(",");
            }
            assertThat("условия показа отправлены в БК с соответствующим значением Geo",
                    context.getGeo(), containsInAnyOrder(resContextGeoList));
        }
    }

    @Test
    @Title("Отправка в БК баннера с учетом минус-регионов, проверяем состояние записи в БД banners_minus_geo[type=bs_synced]")
    public void testBannerBsSyncedMinusGeo() {
        BannersMinusGeoRecord minusGeo = dbSteps.bannersSteps().getBannersMinusGeo(bid, BannersMinusGeoType.bs_synced);
        if (resBannerBsSyncedMinusGeo == null) {
            assertThat("баннер после отправки в БК, в БД находится с соответствующими минус-регионами типа bs_synced",
                    minusGeo, nullValue());
        } else {
            assertThat("баннер после отправки в БК, в БД находится с соответствующими минус-регионами типа bs_synced",
                    minusGeo == null ? null : minusGeo.getMinusGeo(), equalTo(resBannerBsSyncedMinusGeo));
        }
    }

}
