package ru.yandex.autotests.directintapi.bstransport.main.performance.context.filter;

import org.junit.Before;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.cmd.rules.DirectCmdRule;
import ru.yandex.autotests.direct.cmd.rules.PerformanceBannersRule;
import ru.yandex.autotests.direct.db.models.jooq.ppc.enums.BidsPerformanceTargetFunnel;
import ru.yandex.autotests.direct.db.models.jooq.ppc.tables.records.BidsPerformanceRecord;
import ru.yandex.autotests.directapi.darkside.Logins;
import ru.yandex.autotests.directapi.darkside.connection.Semaphore;
import ru.yandex.autotests.directapi.darkside.model.RunBsTransportScriptResponse;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Campaign;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Context;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Filter;
import ru.yandex.autotests.directapi.darkside.steps.DarkSideSteps;
import ru.yandex.autotests.directapi.darkside.steps.TransportSteps;
import ru.yandex.autotests.directapi.darkside.steps.fakesteps.ClientFakeSteps;
import ru.yandex.autotests.direct.utils.tags.TagDictionary;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.autotests.directintapi.bstransport.FeatureNames;
import ru.yandex.autotests.directintapi.bstransport.TransportHelpSteps;
import ru.yandex.autotests.irt.testutils.beandiffer2.comparestrategy.defaultcomparestrategy.DefaultCompareStrategies;
import ru.yandex.qatools.Tag;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Issue;
import ru.yandex.qatools.allure.annotations.Issues;
import ru.yandex.qatools.allure.annotations.Step;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

import java.util.Arrays;
import java.util.Collection;

import static org.hamcrest.Matchers.notNullValue;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assertThat;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assumeThat;
import static ru.yandex.autotests.irt.testutils.beandiffer2.BeanDifferMatcher.beanDiffer;

/**
 * Created by buhter on 09.03.2016
 * https://st.yandex-team.ru/TESTIRT-8693
 * https://st.yandex-team.ru/TESTIRT-7394
 * https://st.yandex-team.ru/TESTIRT-10194
 */


@Aqua.Test
@Tag(TagDictionary.RELEASE)
@Features(FeatureNames.SMART_BANNERS)
@Issues({@Issue("https://st.yandex-team.ru/DIRECT-44221"),
        @Issue("https://st.yandex-team.ru/DIRECT-50613")
})
@RunWith(Parameterized.class)
public class BsTransportOfPerformanceFilterOnlyNewAuditoryAndOnlyOfferRetargetingTest {
    private static final String LOGIN = Logins.LOGIN_TRANSPORT;
    @ClassRule
    public static ApiSteps api = new ApiSteps().as(LOGIN);
    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();
    private static PerformanceBannersRule bannersRule = new PerformanceBannersRule().withUlogin(LOGIN);
    @Rule
    public DirectCmdRule cmdRule = DirectCmdRule.defaultRule().as(Logins.LOGIN_SUPER).withRules(bannersRule);

    @Rule
    public Trashman trashman = new Trashman(api);
    @Parameterized.Parameter(0)
    public BidsPerformanceTargetFunnel targetFunnel;
    @Parameterized.Parameter(1)
    public int onlyNewAuditory;
    @Parameterized.Parameter(2)
    public int onlyOfferRetargeting;
    private ClientFakeSteps clientFakeSteps = api.userSteps.clientFakeSteps();
    private TransportSteps transportSteps = new DarkSideSteps().getTransportSteps();
    private TransportHelpSteps helpSteps = new TransportHelpSteps(api);
    private Filter filter;

    @Parameterized.Parameters(name = "Target funnel - {0}, expected OnlyNewAuditory = {1}" +
            ", expected OnlyOfferRetargeting = {2}")
    public static Collection<Object[]> data() {
        Object[][] data = new Object[][]{
                {BidsPerformanceTargetFunnel.new_auditory, 1, 0},
                {BidsPerformanceTargetFunnel.product_page_visit, 0, 1},
                {BidsPerformanceTargetFunnel.same_products, 0, 0}
        };
        return Arrays.asList(data);
    }

    @Before
    @Step("Подготовка тестовых данных")
    public void before() {
        int shard = clientFakeSteps.getUserShard(LOGIN);
        helpSteps.makePerformanceCampaignReadyForSendingToBS(bannersRule, shard);

        BidsPerformanceRecord bidsPerformance = api.userSteps.getDirectJooqDbSteps().useShard(shard)
                .bidsPerformanceSteps().getBidsPerformanceByPid(bannersRule.getGroupId()).get(0);
        assumeThat("в базе найден BidsPerformance", bidsPerformance, notNullValue());
        bidsPerformance.setTargetFunnel(targetFunnel);
        api.userSteps.getDirectJooqDbSteps().useShard(shard)
                .bidsPerformanceSteps().updateBidsPerformance(bidsPerformance);
        RunBsTransportScriptResponse resp = transportSteps.sendNewCampaign(shard, bannersRule.getCampaignId());
        Campaign campaign = transportSteps.getClientDataRequestCampaign(resp, 0, bannersRule.getCampaignId());
        Context context = campaign != null ? campaign.getContext(bannersRule.getGroupId()) : null;
        assumeThat("в БК отправился CONTEXT"
                , context, notNullValue());
        filter = context.getFilter(bidsPerformance.getPerfFilterId().intValue());
        assumeThat("в БК на уровне CONTEXT отправился FILTER для pid=" + bannersRule.getGroupId()
                , filter, notNullValue());
    }

    @Test
    public void performanceContextFilter() {
        Filter expectedFilter = new Filter();
        expectedFilter.setOnlyOfferRetargeting(onlyOfferRetargeting);
        expectedFilter.setOnlyNewAuditory(onlyNewAuditory);
        assertThat("в БК отправлено правильные поля FILTER.OnlyNewAuditory и FILTER.OnlyOfferRetargeting"
                , filter
                , beanDiffer(expectedFilter).useCompareStrategy(DefaultCompareStrategies.onlyExpectedFields())
        );
    }
}
