package ru.yandex.autotests.directintapi.bstransport.main.retargeting.actions.creation.transmit;
import java.util.List;

import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Test;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.utils.tags.TagDictionary;
import ru.yandex.autotests.directapi.darkside.Logins;
import ru.yandex.autotests.directapi.darkside.connection.Semaphore;
import ru.yandex.autotests.directapi.darkside.model.RunBsTransportScriptResponse;
import ru.yandex.autotests.directapi.darkside.model.bslogs.StopFlag;
import ru.yandex.autotests.directapi.darkside.model.bslogs.UpdateInfo;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Banner;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Campaign;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.Context;
import ru.yandex.autotests.directapi.darkside.model.bslogs.clientdata.GoalContext;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directintapi.bstransport.FeatureNames;
import ru.yandex.autotests.directintapi.bstransport.StoriesNames;
import ru.yandex.autotests.directintapi.bstransport.TransportHelpSteps;
import ru.yandex.qatools.Tag;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Stories;
import ru.yandex.qatools.allure.annotations.Title;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

import static ru.yandex.autotests.direct.utils.matchers.BeanEquals.beanEquals;
import static ru.yandex.autotests.irt.testutils.allure.TestSteps.assertThat;

@Aqua.Test
@Tag(TagDictionary.TRUNK)
@Tag(TagDictionary.RELEASE)
@Title("Транспорт: проверка отправки в БК нового ретаргетинга " +
        "в составе синхронизированной группы с ретаргетингом без фраз")
@Stories(StoriesNames.RETARGETING_ACTION_CREATION)
@Features({FeatureNames.RETARGETINGS, FeatureNames.NOT_FOR_FULL_EXPORT})
public class BsTransportOfNewRetargetingInSyncedGroupWithNoPhraseAndRetargetingTest {

    private static final String login = Logins.LOGIN_TRANSPORT;

    @ClassRule
    public static ApiSteps api = new ApiSteps().as(login);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    private static TransportHelpSteps helpSteps = new TransportHelpSteps(api);

    private static int shard;

    private static Long bid;
    private static Long pid;
    private static Long syncedRetId;
    private static Long newRetId;

    private static Context context;
    private static GoalContext syncedGoalContext;
    private static GoalContext newGoalContext;
    private static Banner banner;

    @BeforeClass
    public static void beforeClass() {
        shard = api.userSteps.clientFakeSteps().getUserShard(login);

        List<Long> retCondIds = helpSteps.addRetargetingListsForUserIfNotExists(login);
        Long syncedRetCondId = retCondIds.get(0);
        Long newRetCondId = retCondIds.get(1);

        Long cid = api.userSteps.campaignSteps().addDefaultTextCampaign();
        pid = api.userSteps.adGroupsSteps().addDefaultGroup(cid);
        bid = api.userSteps.adsSteps().addDefaultTextAd(pid);
        api.userSteps.campaignFakeSteps().makeNewCampaignReadyForSendingToBS(cid);
        api.userSteps.groupFakeSteps().makeGroupFullyModerated(pid);
        api.userSteps.bannersFakeSteps().makeBannerFullyModerated(bid);
        syncedRetId = api.userSteps.audienceTargetsSteps().addWithRetargetingList(login, pid, syncedRetCondId);

        api.userSteps.getDarkSideSteps().getTransportSteps().sendNewCampaign(shard, cid);

        newRetId = api.userSteps.audienceTargetsSteps().addWithRetargetingList(login, pid, newRetCondId);

        RunBsTransportScriptResponse resp = api.userSteps.getDarkSideSteps().getTransportSteps().sendCampaign(shard, cid);
        Campaign campaign = api.userSteps.getDarkSideSteps().getTransportSteps().getClientDataRequestCampaign(resp, cid);
        context = campaign != null ? campaign.getContext(pid) : null;
        banner = context != null ? context.getBanner(bid) : null;
        syncedGoalContext = context != null ? context.getGoalContext(syncedRetId) : null;
        newGoalContext = context != null ? context.getGoalContext(newRetId) : null;
    }

    @Test
    @Title("Отправка в БК нового ретаргетинга в синхронизированной группе " +
            "(группа без фраз, с ретаргетингом) - проверка объекта Контекст")
    public void testTransportOfContextObjForNewRetargetingInSyncedGroupWithNoPhraseAndRetargeting() {
        Context expectedContext = helpSteps.buildExpectedContextObjFromDb(pid, shard, UpdateInfo.UPDATE);
        assertThat("отправленные в БК данные новой группы соответствуют ожидаемым",
                context, beanEquals(expectedContext));
    }

    @Test
    @Title("Отправка в БК нового ретаргетинга в синхронизированной группе " +
            "(группа без фраз, с ретаргетингом) - проверка объекта Баннер")
    public void testTransportOfBannerObjForNewRetargetingInSyncedGroupWithNoPhraseAndRetargeting() {
        Banner expectedBanner = helpSteps.buildExpectedBannerObjFromDb(bid, shard, UpdateInfo.SKIP);
        expectedBanner.setStop(StopFlag.RESUME.value());
        assertThat("отправленные в БК данные нового баннера соответствуют ожидаемым",
                banner, beanEquals(expectedBanner));
    }

    @Test
    @Title("Отправка в БК нового ретаргетинга в синхронизированной группе " +
            "(группа без фраз, с ретаргетингом) - проверка объекта Ретаргетинг (синхронизированный ранее)")
    public void testTransportOfSyncedGoalContextObjForNewRetargetingInSyncedGroupWithNoPhraseAndRetargeting() {
        GoalContext expectedSyncedGoalContext =
                helpSteps.buildExpectedRetargetingObjFromDb(syncedRetId, shard, UpdateInfo.UPDATE);
        assertThat("отправленные в БК данные синхронизированного ретаргетинга соответствуют ожидаемым",
                syncedGoalContext, beanEquals(expectedSyncedGoalContext));
    }

    @Test
    @Title("Отправка в БК нового ретаргетинга в синхронизированной группе " +
            "(группа без фраз, с ретаргетингом) - проверка объекта Ретаргетинг (новый)")
    public void testTransportOfNewGoalContextObjForNewRetargetingInSyncedGroupWithNoPhraseAndRetargeting() {
        GoalContext expectedNewGoalContext =
                helpSteps.buildExpectedRetargetingObjFromDb(newRetId, shard, UpdateInfo.UPDATE);
        assertThat("отправленные в БК данные нового ретаргетинга соответствуют ожидаемым",
                newGoalContext, beanEquals(expectedNewGoalContext));
    }
}
