package ru.yandex.autotests.directapi.banners.tags.getbannerstags;

import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.utils.tags.TagDictionary;
import ru.yandex.autotests.directapi.banners.BannerLogins;
import ru.yandex.autotests.directapi.apiclient.config.Semaphore;
import ru.yandex.autotests.directapi.ApiStories;
import ru.yandex.autotests.directapi.banners.BannerFeatures;
import ru.yandex.autotests.directapi.apiclient.errors.AxisError;
import ru.yandex.autotests.directapi.apiclient.errors.AxisErrorDetails;
import ru.yandex.autotests.directapi.apiclient.methods.Method;
import ru.yandex.autotests.directapi.model.banners.tags.BannersRequestInfoMap;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.autotests.irt.testutils.allure.LogSteps;
import ru.yandex.qatools.Tag;
import ru.yandex.qatools.allure.annotations.Description;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Stories;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

import java.util.Arrays;
import java.util.Collection;

import static org.apache.commons.lang3.StringUtils.capitalize;

/**
 * Created by semkagtn on 11/11/14.
 * https://st.yandex-team.ru/TESTIRT-3320
 */
@Aqua.Test
@Features(BannerFeatures.GET_BANNERS_TAGS)
@Stories(ApiStories.TAGS)
@Tag(TagDictionary.TRUNK)
@RunWith(Parameterized.class)
@Description("Получить метки баннеров: ограничение на число элементов " + BannersRequestInfoMap.CAMPAIGN_IDS)
public class GetBannersTagsCampaignIDSLengthRestrictionTest {

    protected LogSteps log = LogSteps.getLogger(this.getClass());

    @ClassRule
    public static ApiSteps api = new ApiSteps().version(104).as(BannerLogins.CLIENT_SELF);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    @Rule
    public Trashman trasher = new Trashman(api);

    @Parameterized.Parameter(value = 0)
    public String description;

    @Parameterized.Parameter(value = 1)
    public int campaignIdsLength;

    @Parameterized.Parameter(value = 2)
    public AxisError expectedError;

    @Parameterized.Parameters(name = "{0}, число кампаний = {1}")
    public static Collection data() {
        return Arrays.asList(new Object[][]{
                {"Максимальное число элементов", BannersRequestInfoMap.MAX_CAMPAIGN_IDS_LENGTH,
                        new AxisError(71, AxisErrorDetails.ARRAY_MUST_CONTAIN_POSITIVE_INTEGER_VALUES,
                                capitalize(BannersRequestInfoMap.CAMPAIGN_IDS))},
                {"Число элементов на единицу больше максимального", BannersRequestInfoMap.MAX_CAMPAIGN_IDS_LENGTH + 1,
                        new AxisError(241, AxisErrorDetails.THE_ARRAY_MUST_CONTAIN_NO_MORE_THAN_N_ELEMENTS,
                                capitalize(BannersRequestInfoMap.CAMPAIGN_IDS),
                                BannersRequestInfoMap.MAX_CAMPAIGN_IDS_LENGTH)}
        });
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("8583")
    public void campaignIdsLengthRestriction() {
        int[] campaigIDS = new int[campaignIdsLength];

        api.userSteps.shouldGetErrorOn(
                Method.GET_BANNERS_TAGS,
                new BannersRequestInfoMap(api.type())
                        .withCampaignIDS(campaigIDS),
                expectedError);
    }
}
