package ru.yandex.autotests.directapi.clients.updateclientinfo;

import org.apache.commons.lang.RandomStringUtils;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.directapi.clients.ClientFeatures;
import ru.yandex.autotests.directapi.apiclient.config.Semaphore;
import ru.yandex.autotests.direct.utils.model.ShardNumbers;
import ru.yandex.autotests.directapi.common.api45mng.APIPort_PortType;
import ru.yandex.autotests.directapi.common.api45mng.ClientInfo;
import ru.yandex.autotests.directapi.model.Logins;
import ru.yandex.autotests.directapi.model.clients.ClientInfoMap;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

import java.util.Arrays;
import java.util.Collection;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.core.IsEqual.equalTo;

/**
 * Created by ginger on 13.01.14.
 *
 * https://jira.yandex-team.ru/browse/DIRECT-27463
 */
@Aqua.Test
@Features(ClientFeatures.UPDATE_CLIENT_INFO)
@RunWith(Parameterized.class)
public class UpdateClientInfoShardingPositiveTest {
    protected Logger log = LogManager.getLogger(this.getClass());

    @ClassRule
    public static ApiSteps api = new ApiSteps().version(104).wsdl(APIPort_PortType.class);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    @Parameterized.Parameter(value = 0)
    public String operator;

    @Parameterized.Parameter(value = 1)
    public String client;
    public ClientInfo clientInfo;
    private String phone;

    @Parameterized.Parameters(name = "Operator = {0}, Client = {1}")
    public static Collection roles() {
        Object[][] data = new Object[][]{
                {Logins.CLIENT_IN_2_SHARD,Logins.CLIENT_IN_2_SHARD},
                {Logins.AGENCY_IN_2_SHARD, Logins.SUB_IN_2_SHARD},
                {Logins.AGENCY_IN_2_SHARD, Logins.SUB_IN_1_SHARD},
                {Logins.AGENCY_YE_DEFAULT, Logins.SUBCLIENT_IN_2_SHARD},
                {Logins.MANAGER_DEFAULT, Logins.SERVICED_CLIENT_IN_2_SHARD},
                {Logins.MANAGER_DEFAULT, Logins.SUBCLIENT_IN_2_SHARD},
                {Logins.SUPER_LOGIN, Logins.CLIENT_IN_2_SHARD},
                {Logins.SUPPORT, Logins.SERVICED_CLIENT_IN_2_SHARD},
        };
        return Arrays.asList(data);
    }

    @BeforeClass
    public static void initTest(){
        api.userSteps.clientFakeSteps().reshardUserWithLock(Logins.AGENCY_IN_2_SHARD, ShardNumbers.EXTRA_SHARD);
        api.userSteps.clientFakeSteps().reshardUserWithLock(Logins.CLIENT_IN_2_SHARD, ShardNumbers.EXTRA_SHARD);
        api.userSteps.clientFakeSteps().reshardUserWithLock(Logins.SUBCLIENT_IN_2_SHARD, ShardNumbers.EXTRA_SHARD);
        api.userSteps.clientFakeSteps().reshardUserWithLock(Logins.SUB_IN_1_SHARD, ShardNumbers.DEFAULT_SHARD);
        api.userSteps.clientFakeSteps().reshardUserWithLock(Logins.SUB_IN_2_SHARD, ShardNumbers.EXTRA_SHARD);
        api.userSteps.clientFakeSteps().reshardUserWithLock(Logins.SERVICED_CLIENT_IN_2_SHARD, ShardNumbers.EXTRA_SHARD);
    }

    @Before
    public void initClient(){
        api.as(Logins.SUPER_LOGIN);
        clientInfo =  api.userSteps.clientSteps().getClientInfo(client);
        phone = RandomStringUtils.randomNumeric(3);
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("5855")
    public void test(){
        api.as(operator).userSteps.clientSteps().updateClientInfo(
                new ClientInfoMap(clientInfo)
                        .withPhone(phone)
        );
        clientInfo = api.userSteps.clientSteps().getClientInfo(client);
        assertThat("ClientInfo не обновилось.",clientInfo.getPhone(),equalTo(phone));
    }
}
