package ru.yandex.autotests.directapi.finance.accountmanagement.deposit;

import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.utils.money.Currency;
import ru.yandex.autotests.direct.utils.money.Money;
import ru.yandex.autotests.direct.utils.money.MoneyCurrency;
import ru.yandex.autotests.direct.utils.tags.TagDictionary;
import ru.yandex.autotests.directapi.apiclient.config.Semaphore;
import ru.yandex.autotests.directapi.apiclient.errors.AxisError;
import ru.yandex.autotests.directapi.apiclient.errors.AxisErrorDetails;
import ru.yandex.autotests.directapi.common.api45.Account;
import ru.yandex.autotests.directapi.finance.FinanceFeatures;
import ru.yandex.autotests.directapi.finance.FinanceLogins;
import ru.yandex.autotests.directapi.model.Logins;
import ru.yandex.autotests.directapi.model.finances.PayMethod;
import ru.yandex.autotests.directapi.model.finances.PaymentMap;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.autotests.irt.testutils.allure.LogSteps;
import ru.yandex.qatools.Tag;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.hazelcast.SemaphoreRule;


/**
 * User: mariabye
 * Date: 17.10.13
 */
@Aqua.Test
@Features(FinanceFeatures.ACCOUNT_MANAGEMENT_DEPOSIT)
@Tag(TagDictionary.TRUNK)
public class DepositYMModeTest {
    private LogSteps log = LogSteps.getLogger(this.getClass());

    static String login = FinanceLogins.DEPOSIT_FREE_CLIENT;

    @ClassRule
    public static ApiSteps api = new ApiSteps().version(104).as(login);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    @Rule
    public Trashman trasher = new Trashman(api);

    private static float minPaymentAmount;

    @BeforeClass
    public static void getAccountIDs() {
        api.as(FinanceLogins.INVOICE_FREE_CLIENT).userSteps.addModeratedCampaign(FinanceLogins.INVOICE_FREE_CLIENT);
        api.as(Logins.SUPER_LOGIN).userSteps.financeSteps().verifySharedAccount(FinanceLogins.INVOICE_FREE_CLIENT);
        api.as(FinanceLogins.DEPOSIT_FREE_CLIENT).userSteps.addModeratedCampaign(FinanceLogins.DEPOSIT_FREE_CLIENT);
        api.as(Logins.SUPER_LOGIN).userSteps.financeSteps().verifySharedAccount(FinanceLogins.DEPOSIT_FREE_CLIENT);
        minPaymentAmount = MoneyCurrency.get(Currency.RUB).getMinInvoiceAmount().floatValue();
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("6009")
    public void payAccountWithFinanceToken() {
        Account account = api.as(login).userSteps.financeSteps().getAccount(login);
        api.userSteps.financeSteps().shouldGetErrorsOnAMDeposit(
                new PaymentMap[]{new PaymentMap(api.type())
                        .withAccountID(account.getAccountID())
                        .withOrigin(PayMethod.YM)
                        .withAmount(Money.valueOf(minPaymentAmount, Currency.RUB).addVAT().floatValue())
                        .withCurrency(Currency.RUB)},
                new AxisError(360, AxisErrorDetails.EMPTY_STRING));
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("6008")
    public void payAccountYMNotAvailable() {
        Account account = api.as(FinanceLogins.INVOICE_FREE_CLIENT).userSteps.financeSteps().getAccount(
                FinanceLogins.INVOICE_FREE_CLIENT
        );
        api.userSteps.financeSteps().shouldGetErrorsOnAMDeposit(
                new PaymentMap[]{new PaymentMap(api.type())
                        .withAccountID(account.getAccountID())
                        .withOrigin(PayMethod.YM)
                        .withAmount(minPaymentAmount)
                        .withCurrency(Currency.RUB)},
                new AxisError(360, AxisErrorDetails.EMPTY_STRING));
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("6010")
    public void payAccountByManagerNotAvailable() {
        Account account = api.as(login).userSteps.financeSteps().getAccount(login);
        api.userSteps.financeSteps().shouldGetErrorsOnAMDeposit(
                new PaymentMap[]{new PaymentMap(api.type())
                        .withAccountID(account.getAccountID())
                        .withOrigin(PayMethod.YM)
                        .withAmount(minPaymentAmount)
                        .withCurrency(Currency.RUB)},
                new AxisError(360, AxisErrorDetails.EMPTY_STRING));
    }
}
