package ru.yandex.autotests.directapi.finance.accountmanagement.transfer;

import java.util.Arrays;
import java.util.Collection;

import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.direct.utils.model.ShardNumbers;
import ru.yandex.autotests.direct.utils.money.Currency;
import ru.yandex.autotests.direct.utils.money.Money;
import ru.yandex.autotests.direct.utils.money.MoneyCurrency;
import ru.yandex.autotests.directapi.apiclient.config.Semaphore;
import ru.yandex.autotests.directapi.apiclient.errors.AxisError;
import ru.yandex.autotests.directapi.apiclient.errors.AxisErrorDetails;
import ru.yandex.autotests.directapi.common.api45.Account;
import ru.yandex.autotests.directapi.finance.FinanceFeatures;
import ru.yandex.autotests.directapi.finance.FinanceLogins;
import ru.yandex.autotests.directapi.model.Logins;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.autotests.irt.testutils.allure.LogSteps;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

/**
 * User: pavryabov
 * Date: 22.01.14
 * <p>
 * https://jira.yandex-team.ru/browse/TESTIRT-1429
 */
@Aqua.Test
@Features(FinanceFeatures.ACCOUNT_MANAGEMENT_TRANSFER)
@RunWith(Parameterized.class)
public class AccountManagementTransferShardingNegativeTest {
    private LogSteps log = LogSteps.getLogger(this.getClass());

    @ClassRule
    public static ApiSteps api = new ApiSteps().version(104);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    @Rule
    public Trashman trasher = new Trashman(api);

    @Parameterized.Parameter(value = 0)
    public String operator;

    @Parameterized.Parameter(value = 1)
    public String clientFrom;

    @Parameterized.Parameter(value = 2)
    public String clientTo;

    @Parameterized.Parameter(value = 3)
    public String accountElseOwner;

    @Parameterized.Parameters(name = "Operator = {0}, ClientFrom = {1}, ClientTo = {2}")
    public static Collection strategies() {
        Object[][] data = new Object[][]{
                {Logins.AGENCY_SHARD_2, Logins.ACCOUNT_SUBCLIENT_SHARD, FinanceLogins.AGENCY_AND_FREE_CLIENT_YE,
                        Logins.AGENCY_YE_DEFAULT},
                {Logins.AGENCY_YE_DEFAULT, FinanceLogins.AGENCY_AND_FREE_CLIENT_YE, Logins.ACCOUNT_SUBCLIENT_SHARD,
                        Logins.AGENCY_SHARD_2}
        };
        return Arrays.asList(data);
    }

    public Account accountFrom;
    public Account accountTo;
    public Money amount = MoneyCurrency.get(null).getMinTransferAmount();

    @BeforeClass
    public static void reshard() {
        api.userSteps.clientFakeSteps().enableAgencyWallet(Logins.AGENCY_YE_DEFAULT);
        api.as(Logins.AGENCY_YE_DEFAULT).userSteps.createAccount(FinanceLogins.AGENCY_AND_FREE_CLIENT_YE);
        api.userSteps.clientFakeSteps().enableAgencyWallet(Logins.AGENCY_SHARD_2);
        api.as(Logins.AGENCY_SHARD_2).userSteps.createAccount(Logins.ACCOUNT_SUBCLIENT_SHARD);
        api.userSteps.clientFakeSteps().reshardUserWithLock(Logins.AGENCY_SHARD_2, ShardNumbers.EXTRA_SHARD);
        api.userSteps.clientFakeSteps().reshardUserWithLock(Logins.ACCOUNT_SUBCLIENT_SHARD, ShardNumbers.EXTRA_SHARD);
    }

    @Before
    public void initTest() {
        accountFrom = api.as(operator).userSteps.financeSteps().getAccount(clientFrom);
        accountTo = api.as(accountElseOwner).userSteps.financeSteps().getAccount(clientTo);
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("6095")
    public void accountManagementTransferTest() {
        api.as(operator).userSteps.financeSteps().shouldGetMultipleErrorOnTransfer(
                accountFrom.getAccountID(),
                accountTo.getAccountID(),
                amount.floatValue(),
                null,
                new AxisError(54, AxisErrorDetails.ACCOUNT, accountTo.getAccountID()));
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("6096")
    public void accountManagementReverseTransferTest() {
        api.as(operator).userSteps.financeSteps().shouldGetMultipleErrorOnTransfer(
                accountTo.getAccountID(),
                accountFrom.getAccountID(),
                amount.floatValue(),
                null,
                new AxisError(54, AxisErrorDetails.ACCOUNT, accountTo.getAccountID()));
    }
}
