package ru.yandex.autotests.directapi.finance.checkpayment;

import java.util.Arrays;

import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;

import ru.yandex.aqua.annotations.project.Aqua;
import ru.yandex.autotests.directapi.apiclient.config.Semaphore;
import ru.yandex.autotests.directapi.apiclient.errors.AxisError;
import ru.yandex.autotests.directapi.apiclient.methods.Method;
import ru.yandex.autotests.directapi.finance.FinanceFeatures;
import ru.yandex.autotests.directapi.model.Logins;
import ru.yandex.autotests.directapi.model.finances.CheckPaymentInfoMap;
import ru.yandex.autotests.directapi.model.finances.PayCampaignsByCardInfoMap;
import ru.yandex.autotests.directapi.rules.ApiSteps;
import ru.yandex.autotests.directapi.rules.Trashman;
import ru.yandex.qatools.allure.annotations.Description;
import ru.yandex.qatools.allure.annotations.Features;
import ru.yandex.qatools.allure.annotations.Issue;
import ru.yandex.qatools.hazelcast.SemaphoreRule;

import static org.apache.commons.lang3.RandomStringUtils.randomAlphabetic;
import static org.apache.commons.lang3.RandomStringUtils.randomAlphanumeric;
import static org.apache.commons.lang3.RandomStringUtils.randomNumeric;
import static org.apache.commons.lang3.StringUtils.lowerCase;
import static org.apache.commons.lang3.StringUtils.upperCase;

/**
 * Created by semkagtn on 28.09.15.
 * https://st.yandex-team.ru/TESTIRT-6408
 */
@Aqua.Test
@Features(FinanceFeatures.CHECK_PAYMENT)
@Description("Проверка валидации поля CustomTransactionId в запросе: позитивные проверки")
@Issue("https://st.yandex-team.ru/DIRECT-42633")
@RunWith(Parameterized.class)
public class CheckPaymentCustomTransactionIdValidationPositiveTest {

    private static final String LOGIN = Logins.CLIENT_FREE_YE_DEFAULT;

    @ClassRule
    public static ApiSteps api = new ApiSteps().version(104).as(LOGIN);

    @Rule
    public Trashman trasher = new Trashman(api);

    @ClassRule
    public static SemaphoreRule semaphore = Semaphore.getSemaphore();

    @Parameterized.Parameter(0)
    public String description;

    @Parameterized.Parameter(1)
    public String customTransactionId;

    @Parameterized.Parameters(name = "{0}: CustomTransactionID = {1}")
    public static Iterable<Object[]> data() {
        return Arrays.asList(new Object[][]{
                {"только цифры", randomNumeric(PayCampaignsByCardInfoMap.CUSTOM_TRANSACTION_ID_LENGTH)},
                {"только маленькие буквы",
                        lowerCase(randomAlphabetic(PayCampaignsByCardInfoMap.CUSTOM_TRANSACTION_ID_LENGTH))},
                {"только большие буквы",
                        upperCase(randomAlphabetic(PayCampaignsByCardInfoMap.CUSTOM_TRANSACTION_ID_LENGTH))},
                {"маленькие и большие буквы", randomAlphabetic(PayCampaignsByCardInfoMap.CUSTOM_TRANSACTION_ID_LENGTH)},
                {"буквы и цифры", randomAlphanumeric(PayCampaignsByCardInfoMap.CUSTOM_TRANSACTION_ID_LENGTH)},
        });
    }

    @Test
    @ru.yandex.qatools.allure.annotations.TestCaseId("6151")
    public void invalidCustomTransactionIdInRequest() {
        api.userSteps.financeSteps().shouldGetErrorOn(Method.CHECK_PAYMENT,
                new CheckPaymentInfoMap<>(api.type())
                        .withCustomTransactionId(customTransactionId),
                new AxisError(370));
    }
}
